@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,22 @@
 #include <charconv>
 #include <wx/translation.h>
 
+
+static void removeTrailingZeros( wxString& aText )
+{
+    int len = aText.length();
+    int removeLast = 0;
+
+    while( --len > 0 && aText[len] == '0' )
+        removeLast++;
+
+    if( len >= 0 && ( aText[len] == '.' || aText[len] == ',' ) )
+        removeLast++;
+
+    aText = aText.RemoveLast( removeLast );
+}
+
+
 bool EDA_UNIT_UTILS::IsImperialUnit( EDA_UNITS aUnit )
 {
     switch( aUnit )
@@ -103,6 +119,7 @@ bool EDA_UNIT_UTILS::FetchUnitsFromString( const wxString& aTextValue, EDA_UNITS
         aUnits = EDA_UNITS::DEGREES;
     else
         return false;
+
     return true;
 }
 
@@ -240,6 +257,7 @@ bool EDA_UNIT_UTILS::ParseInternalUnits( const std::string& aInput, const EDA_IU
 #define IN_TO_IU( x, scale ) ( x * scale.IU_PER_MILS * 1000 )
 #define MILS_TO_IU( x, scale ) ( x * scale.IU_PER_MILS )
 
+
 double EDA_UNIT_UTILS::UI::ToUserUnit( const EDA_IU_SCALE& aIuScale, EDA_UNITS aUnit,
                                        double aValue )
 {
@@ -269,23 +287,12 @@ double EDA_UNIT_UTILS::UI::ToUserUnit( const EDA_IU_SCALE& aIuScale, EDA_UNITS a
 }
 
 
-/**
- * Convert a value to a string using double notation.
- *
- * For readability, the mantissa has 3 or more digits,
- * the trailing 0 are removed if the mantissa has more than 3 digits
- * and some trailing 0
- * This function should be used to display values in dialogs because a value
- * entered in mm (for instance 2.0 mm) could need up to 8 digits mantissa
- * if displayed in inch to avoid truncation or rounding made just by the printf function.
- * otherwise the actual value is rounded when read from dialog and converted
- * in internal units, and therefore modified.
- */
 wxString EDA_UNIT_UTILS::UI::StringFromValue( const EDA_IU_SCALE& aIuScale, EDA_UNITS aUnits,
                                               double aValue, bool aAddUnitsText,
                                               EDA_DATA_TYPE aType )
 {
     double value_to_print = aValue;
+    bool   is_eeschema = ( aIuScale.IU_PER_MM == SCH_IU_PER_MM );
 
     switch( aType )
     {
@@ -305,46 +312,46 @@ wxString EDA_UNIT_UTILS::UI::StringFromValue( const EDA_IU_SCALE& aIuScale, EDA_
         break;
     }
 
-    char buf[50];
+    const wxChar* format = nullptr;
 
-    if( value_to_print != 0.0 && fabs( value_to_print ) <= 0.0001 )
+    switch( aUnits )
     {
-        int len = snprintf( buf, sizeof( buf ) - 1, "%.10f", value_to_print );
 
-        while( --len > 0 && buf[len] == '0' )
-            buf[len] = '\0';
+    case EDA_UNITS::MILS:
+        format = is_eeschema ? wxT( "%.3f" ) : wxT( "%.5f" );
+        break;
 
-        if( len >= 0 && ( buf[len] == '.' || buf[len] == ',' ) )
-            buf[len] = '\0';
-    }
-    else
-    {
-        snprintf( buf, sizeof( buf ) - 1, "%.10g", value_to_print );
+    case EDA_UNITS::INCHES:
+        format = is_eeschema ? wxT( "%.6f" ) : wxT( "%.8f" );
+        break;
+
+    case EDA_UNITS::DEGREES:
+        format = wxT( "%.4f" );
+        break;
+
+    default:
+        format = wxT( "%.10f" );
+        break;
     }
 
-    wxString stringValue( buf, wxConvUTF8 );
+    wxString text;
+    text.Printf( format, value_to_print );
+    removeTrailingZeros( text );
+
+    if( value_to_print != 0.0 && ( text == wxS( "0" ) || text == wxS( "-0" ) ) )
+    {
+        text.Printf( wxS( "%.10f" ), value_to_print );
+        removeTrailingZeros( text );
+    }
 
     if( aAddUnitsText )
-        stringValue += EDA_UNIT_UTILS::GetText( aUnits, aType );
+        text << EDA_UNIT_UTILS::GetText( aUnits, aType );
 
-    return stringValue;
+    return text;
 }
 
 
 
-/**
- * Convert a value to a string using double notation.
- *
- * For readability, the mantissa has 0, 1, 3 or 4 digits, depending on units
- * for unit = inch the mantissa has 3 digits (Eeschema) or 4 digits
- * for unit = mil the mantissa has 0 digits (Eeschema) or 1 digits
- * for unit = mm the mantissa has 3 digits (Eeschema) or 4 digits
- * Should be used only to display info in status,
- * but not in dialogs, because 4 digits only
- * could truncate the actual value
- */
-
-
 // A lower-precision (for readability) version of StringFromValue()
 wxString EDA_UNIT_UTILS::UI::MessageTextFromValue( const EDA_IU_SCALE& aIuScale, EDA_UNITS aUnits,
                                                    int aValue,
@@ -382,6 +389,7 @@ wxString EDA_UNIT_UTILS::UI::MessageTextFromValue( const EDA_IU_SCALE& aIuScale,
     wxString      text;
     const wxChar* format;
     double        value = aValue;
+    bool          is_eeschema = ( aIuScale.IU_PER_MM == SCH_IU_PER_MM );
 
     switch( aType )
     {
@@ -407,43 +415,23 @@ wxString EDA_UNIT_UTILS::UI::MessageTextFromValue( const EDA_IU_SCALE& aIuScale,
     {
     default:
     case EDA_UNITS::MICROMETRES:
-#if defined( EESCHEMA )
-        format = wxT( "%.0f" );
-#else
-        format = wxT( "%.1f" );
-#endif
+        format = is_eeschema ? wxT( "%.0f" ) : wxT( "%.1f" );
         break;
 
     case EDA_UNITS::MILLIMETRES:
-#if defined( EESCHEMA )
-        format = wxT( "%.2f" );
-#else
-        format = wxT( "%.4f" );
-#endif
+        format = is_eeschema ? wxT( "%.2f" ) : wxT( "%.4f" );
         break;
 
     case EDA_UNITS::CENTIMETRES:
-#if defined( EESCHEMA )
-        format = wxT( "%.3f" );
-#else
-        format = wxT( "%.5f" );
-#endif
+        format = is_eeschema ? wxT( "%.3f" ) : wxT( "%.5f" );
         break;
 
     case EDA_UNITS::MILS:
-#if defined( EESCHEMA )
-        format = wxT( "%.0f" );
-#else
-        format = wxT( "%.2f" );
-#endif
+        format = is_eeschema ? wxT( "%.0f" ) : wxT( "%.2f" );
         break;
 
     case EDA_UNITS::INCHES:
-#if defined( EESCHEMA )
-        format = wxT( "%.3f" );
-#else
-        format = wxT( "%.4f" );
-#endif
+        format = is_eeschema ? wxT( "%.3f" ) : wxT( "%.4f" );
         break;
 
     case EDA_UNITS::DEGREES:
