@@ -2,7 +2,7 @@
  * This program source code file is part of KICAD, a free EDA CAD application.
  *
  * Copyright (C) 2021 Ola Rinta-Koski
- * Copyright (C) 2021-2023 Kicad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * Font abstract base class
  *
@@ -57,7 +57,7 @@ const METRICS& METRICS::Default()
 
 FONT* FONT::s_defaultFont = nullptr;
 
-std::map< std::tuple<wxString, bool, bool>, FONT*> FONT::s_fontMap;
+std::map< std::tuple<wxString, bool, bool, bool>, FONT*> FONT::s_fontMap;
 
 class MARKUP_CACHE
 {
@@ -127,6 +127,7 @@ private:
 
 static MARKUP_CACHE s_markupCache( 1024 );
 static std::mutex s_markupCacheMutex;
+static std::mutex s_defaultFontMutex;;
 
 
 FONT::FONT()
@@ -136,6 +137,8 @@ FONT::FONT()
 
 FONT* FONT::getDefaultFont()
 {
+    std::lock_guard lock( s_defaultFontMutex );
+
     if( !s_defaultFont )
         s_defaultFont = STROKE_FONT::LoadFont( wxEmptyString );
 
@@ -143,12 +146,13 @@ FONT* FONT::getDefaultFont()
 }
 
 
-FONT* FONT::GetFont( const wxString& aFontName, bool aBold, bool aItalic )
+FONT* FONT::GetFont( const wxString& aFontName, bool aBold, bool aItalic,
+                     const std::vector<wxString>* aEmbeddedFiles, bool aForDrawingSheet )
 {
     if( aFontName.empty() || aFontName.StartsWith( KICAD_FONT_NAME ) )
         return getDefaultFont();
 
-    std::tuple<wxString, bool, bool> key = { aFontName, aBold, aItalic };
+    std::tuple<wxString, bool, bool, bool> key = { aFontName, aBold, aItalic, aForDrawingSheet };
 
     FONT* font = nullptr;
 
@@ -156,7 +160,8 @@ FONT* FONT::GetFont( const wxString& aFontName, bool aBold, bool aItalic )
         font = s_fontMap[key];
 
     if( !font )
-        font = OUTLINE_FONT::LoadFont( aFontName, aBold, aItalic );
+        font = OUTLINE_FONT::LoadFont( aFontName, aBold, aItalic, aEmbeddedFiles,
+                                       aForDrawingSheet );
 
     if( !font )
         font = getDefaultFont();
@@ -217,6 +222,9 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
     case GR_TEXT_V_ALIGN_TOP:                            break;
     case GR_TEXT_V_ALIGN_CENTER: offset.y -= height / 2; break;
     case GR_TEXT_V_ALIGN_BOTTOM: offset.y -= height;     break;
+    case GR_TEXT_V_ALIGN_INDETERMINATE:
+        wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+        break;
     }
 
     for( int i = 0; i < lineCount; i++ )
@@ -228,9 +236,12 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
 
         switch( aAttrs.m_Halign )
         {
-        case GR_TEXT_H_ALIGN_LEFT:                                   break;
-        case GR_TEXT_H_ALIGN_CENTER: lineOffset.x = -lineSize.x / 2; break;
-        case GR_TEXT_H_ALIGN_RIGHT:  lineOffset.x = -( lineSize.x + offset.x );     break;
+        case GR_TEXT_H_ALIGN_LEFT:                                              break;
+        case GR_TEXT_H_ALIGN_CENTER: lineOffset.x = -lineSize.x / 2;            break;
+        case GR_TEXT_H_ALIGN_RIGHT:  lineOffset.x = -( lineSize.x + offset.x ); break;
+        case GR_TEXT_H_ALIGN_INDETERMINATE:
+            wxFAIL_MSG( wxT( "Indeterminate state legal only in dialogs." ) );
+            break;
         }
 
         aPositions.push_back( aPosition + lineOffset );
@@ -238,16 +249,6 @@ void FONT::getLinePositions( const wxString& aText, const VECTOR2I& aPosition,
 }
 
 
-/**
- * Draw a string.
- *
- * @param aGal
- * @param aText is the text to be drawn.
- * @param aPosition is the text object position in world coordinates.
- * @param aCursor is the current text position (for multiple text blocks within a single text
- *                object, such as a run of superscript characters)
- * @param aAttrs are the styling attributes of the text, including its rotation
- */
 void FONT::Draw( KIGFX::GAL* aGal, const wxString& aText, const VECTOR2I& aPosition,
                  const VECTOR2I& aCursor, const TEXT_ATTRIBUTES& aAttrs,
                  const METRICS& aFontMetrics ) const
@@ -276,7 +277,7 @@ void FONT::Draw( KIGFX::GAL* aGal, const wxString& aText, const VECTOR2I& aPosit
 
 
 /**
- * @return position of cursor for drawing next substring
+ * @return position of cursor for drawing next substring.
  */
 VECTOR2I drawMarkup( BOX2I* aBoundingBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
                      const MARKUP::NODE* aNode, const VECTOR2I& aPosition,
@@ -471,9 +472,11 @@ VECTOR2I FONT::boundingBoxSingleLine( BOX2I* aBBox, const wxString& aText,
 }
 
 
-/*
- * Break marked-up text into "words".  In this context, a "word" is EITHER a run of marked-up
- * text (subscript, superscript or overbar), OR a run of non-marked-up text separated by spaces.
+/**
+ * Break marked-up text into "words".
+ *
+ * In this context, a "word" is EITHER a run of marked-up text (subscript, superscript or
+ * overbar), OR a run of non-marked-up text separated by spaces.
  */
 void wordbreakMarkup( std::vector<std::pair<wxString, int>>* aWords,
                       const std::unique_ptr<MARKUP::NODE>& aNode, const KIFONT::FONT* aFont,
@@ -568,17 +571,6 @@ void FONT::wordbreakMarkup( std::vector<std::pair<wxString, int>>* aWords, const
 }
 
 
-/*
- * This is a highly simplified line-breaker.  KiCad is an EDA tool, not a word processor.
- *
- * 1) It breaks only on spaces.  If you type a word wider than the column width then you get
- *    overflow.
- * 2) It treats runs of formatted text (superscript, subscript, overbar) as single words.
- * 3) It does not perform justification.
- *
- * The results of the linebreaking are the addition of \n in the text.  It is presumed that this
- * function is called on m_shownText (or equivalent) rather than the original source text.
- */
 void FONT::LinebreakText( wxString& aText, int aColumnWidth, const VECTOR2I& aSize, int aThickness,
                           bool aBold, bool aItalic ) const
 {
