@@ -7,7 +7,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 1992-2012 Lorenzo Marcantonio, l.marcantonio@logossrl.com
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,9 +35,11 @@
 #include <wx/zstream.h>
 #include <wx/wfstream.h>
 #include <wx/datstrm.h>
+#include <wx/tokenzr.h>
 
 #include <advanced_config.h>
-#include <eda_text.h> // for IsGotoPageHref
+#include <common.h>               // ResolveUriByEnvVars
+#include <eda_text.h>             // for IsGotoPageHref
 #include <font/font.h>
 #include <core/ignore.h>
 #include <macros.h>
@@ -354,21 +356,21 @@ void PDF_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
     SetCurrentLineWidth( aWidth );
 
     // Usual trig arc plotting routine...
-    start.x = aCenter.x + KiROUND( aRadius * (-startAngle).Cos() );
-    start.y = aCenter.y + KiROUND( aRadius * (-startAngle).Sin() );
+    start.x = KiROUND( aCenter.x + aRadius * ( -startAngle ).Cos() );
+    start.y = KiROUND( aCenter.y + aRadius * ( -startAngle ).Sin() );
     VECTOR2D pos_dev = userToDeviceCoordinates( start );
     fprintf( m_workFile, "%g %g m ", pos_dev.x, pos_dev.y );
 
     for( EDA_ANGLE ii = startAngle + delta; ii < endAngle; ii += delta )
     {
-        end.x = aCenter.x + KiROUND( aRadius * (-ii).Cos() );
-        end.y = aCenter.y + KiROUND( aRadius * (-ii).Sin() );
+        end.x = KiROUND( aCenter.x + aRadius * ( -ii ).Cos() );
+        end.y = KiROUND( aCenter.y + aRadius * ( -ii ).Sin() );
         pos_dev = userToDeviceCoordinates( end );
         fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
     }
 
-    end.x = aCenter.x + KiROUND( aRadius * (-endAngle).Cos() );
-    end.y = aCenter.y + KiROUND( aRadius * (-endAngle).Sin() );
+    end.x = KiROUND( aCenter.x + aRadius * ( -endAngle ).Cos() );
+    end.y = KiROUND( aCenter.y + aRadius * ( -endAngle ).Sin() );
     pos_dev = userToDeviceCoordinates( end );
     fprintf( m_workFile, "%g %g l ", pos_dev.x, pos_dev.y );
 
@@ -400,12 +402,12 @@ void PDF_PLOTTER::PlotPoly( const std::vector<VECTOR2I>& aCornerList, FILL_T aFi
     SetCurrentLineWidth( aWidth );
 
     VECTOR2D pos = userToDeviceCoordinates( aCornerList[0] );
-    fprintf( m_workFile, "%g %g m\n", pos.x, pos.y );
+    fprintf( m_workFile, "%f %f m\n", pos.x, pos.y );
 
     for( unsigned ii = 1; ii < aCornerList.size(); ii++ )
     {
         pos = userToDeviceCoordinates( aCornerList[ii] );
-        fprintf( m_workFile, "%g %g l\n", pos.x, pos.y );
+        fprintf( m_workFile, "%f %f l\n", pos.x, pos.y );
     }
 
     // Close path and stroke and/or fill
@@ -438,7 +440,7 @@ void PDF_PLOTTER::PenTo( const VECTOR2I& pos, char plume )
     if( m_penState != plume || pos != m_penLastpos )
     {
         VECTOR2D pos_dev = userToDeviceCoordinates( pos );
-        fprintf( m_workFile, "%g %g %c\n",
+        fprintf( m_workFile, "%f %f %c\n",
                  pos_dev.x, pos_dev.y,
                  ( plume=='D' ) ? 'l' : 'm' );
     }
@@ -480,9 +482,10 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
             if( image.HasAlpha() != aCurrImage.HasAlpha() )
                 continue;
 
-            if( image.HasMask() != aCurrImage.HasMask() || image.GetMaskRed() != aCurrImage.GetMaskRed()
-                || image.GetMaskGreen() != aCurrImage.GetMaskGreen()
-                || image.GetMaskBlue() != aCurrImage.GetMaskBlue() )
+            if( image.HasMask() != aCurrImage.HasMask()
+              || image.GetMaskRed() != aCurrImage.GetMaskRed()
+              || image.GetMaskGreen() != aCurrImage.GetMaskGreen()
+              || image.GetMaskBlue() != aCurrImage.GetMaskBlue() )
                 continue;
 
             int pixCount = image.GetWidth() * image.GetHeight();
@@ -490,7 +493,8 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
             if( memcmp( image.GetData(), aCurrImage.GetData(), pixCount * 3 ) != 0 )
                 continue;
 
-            if( image.HasAlpha() && memcmp( image.GetAlpha(), aCurrImage.GetAlpha(), pixCount ) != 0 )
+            if( image.HasAlpha()
+              && memcmp( image.GetAlpha(), aCurrImage.GetAlpha(), pixCount ) != 0 )
                 continue;
 
             return imgHandle;
@@ -773,9 +777,21 @@ void PDF_PLOTTER::ClosePage()
     auto iuToPdfUserSpace =
             [&]( const VECTOR2I& aCoord ) -> VECTOR2D
             {
-                VECTOR2D retval = VECTOR2D( aCoord ) * PTsPERMIL / ( m_IUsPerDecimil * 10 );
+                VECTOR2D pos = VECTOR2D( aCoord ) * PTsPERMIL / ( m_IUsPerDecimil * 10 );
+
                 // PDF y=0 is at bottom of page, invert coordinate
-                retval.y = psPaperSize.y - retval.y;
+                VECTOR2D retval( pos.x, psPaperSize.y - pos.y );
+
+                // The pdf plot can be mirrored (from left to right). So mirror the
+                // x coordinate if m_plotMirror is set
+                if( m_plotMirror )
+                {
+                    if( m_mirrorIsHorizontal )
+                        retval.x = ( psPaperSize.x - pos.x );
+                    else
+                        retval.y = pos.y;
+                }
+
                 return retval;
             };
 
@@ -884,7 +900,6 @@ void PDF_PLOTTER::ClosePage()
         pageOutlineName = wxString::Format( _( "%s (Page %s)" ), m_pageName, m_pageNumbers.back() );
     }
 
-
     int           actionHandle = emitGoToAction( pageHandle );
     OUTLINE_NODE* pageOutlineNode =
             addOutlineNode( m_outlineRoot.get(), actionHandle, pageOutlineName );
@@ -1108,7 +1123,9 @@ int PDF_PLOTTER::emitOutline()
 
 bool PDF_PLOTTER::EndPlot()
 {
-    wxASSERT( m_outputFile );
+    // We can end up here if there was nothing to plot
+    if( !m_outputFile )
+        return false;
 
     // Close the current page (often the only one)
     ClosePage();
@@ -1332,16 +1349,35 @@ bool PDF_PLOTTER::EndPlot()
                 {
                     wxString href = property.substr( property.Find( "http:" ) );
 
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
                     js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
-                                            EscapeString( property, CTX_JS_STR ),
+                                            EscapeString( href, CTX_JS_STR ),
                                             EscapeString( href, CTX_JS_STR ) );
                 }
                 else if( property.Find( "https:" ) >= 0 )
                 {
                     wxString href = property.substr( property.Find( "https:" ) );
 
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
                     js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
-                                            EscapeString( property, CTX_JS_STR ),
+                                            EscapeString( href, CTX_JS_STR ),
+                                            EscapeString( href, CTX_JS_STR ) );
+                }
+                else if( property.Find( "file:" ) >= 0 )
+                {
+                    wxString href = property.substr( property.Find( "file:" ) );
+
+                    if( m_project )
+                        href = ResolveUriByEnvVars( href, m_project );
+
+                    href = NormalizeFileUri( href );
+
+                    js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
+                                            EscapeString( href, CTX_JS_STR ),
                                             EscapeString( href, CTX_JS_STR ) );
                 }
                 else
@@ -1367,13 +1403,22 @@ bool PDF_PLOTTER::EndPlot()
                     }
                 }
             }
-            else if( url.StartsWith( "http:" ) || url.StartsWith( "https:" ) )
+            else if( url.StartsWith( "http:" ) || url.StartsWith( "https:" )
+                   || url.StartsWith( "file:" ) )
             {
-                wxString menuText = wxString::Format( _( "Open %s" ), url );
+                wxString href = url;
+
+                if( m_project )
+                    href = ResolveUriByEnvVars( url, m_project );
+
+                if( url.StartsWith( "file:" ) )
+                    href = NormalizeFileUri( href );
+
+                wxString menuText = wxString::Format( _( "Open %s" ), href );
 
                 js += wxString::Format( wxT( "[\"%s\", \"%s\"],\n" ),
-                                        EscapeString( menuText, CTX_JS_STR ),
-                                        EscapeString( url, CTX_JS_STR ) );
+                                        EscapeString( href, CTX_JS_STR ),
+                                        EscapeString( href, CTX_JS_STR ) );
             }
         }
 
@@ -1413,6 +1458,7 @@ function ShM(aEntries) {
     var cChoice = app.popUpMenuEx.apply(app, aParams);
     if (cChoice != null && cChoice.substring(0, 1) == '#') this.pageNum = parseInt(cChoice.slice(1));
     else if (cChoice != null && cChoice.substring(0, 4) == 'http') app.launchURL(cChoice);
+    else if (cChoice != null && cChoice.substring(0, 4) == 'file') app.openDoc(cChoice.substring(7));
 }
 )JS";
 
@@ -1444,7 +1490,6 @@ function ShM(aEntries) {
             ">>\n", (long) m_pageHandles.size() );
     closePdfObject();
 
-
     // The info dictionary
     int infoDictHandle = startPdfObject();
     char date_buf[250];
@@ -1463,10 +1508,14 @@ function ShM(aEntries) {
              "/Producer (KiCad PDF)\n"
              "/CreationDate (%s)\n"
              "/Creator %s\n"
-             "/Title %s\n",
+             "/Title %s\n"
+             "/Author %s\n"
+             "/Subject %s\n",
              date_buf,
              encodeStringForPlotter( m_creator ).c_str(),
-             encodeStringForPlotter( m_title ).c_str() );
+             encodeStringForPlotter( m_title ).c_str(),
+             encodeStringForPlotter( m_author ).c_str(),
+             encodeStringForPlotter( m_subject ).c_str() );
 
     fputs( ">>\n", m_outputFile );
     closePdfObject();
@@ -1634,7 +1683,7 @@ void PDF_PLOTTER::Text( const VECTOR2I&        aPos,
            coordinate system will be used for the overlining. Also the %f
            for the trig part of the matrix to avoid %g going in exponential
            format (which is not supported) */
-        fprintf( m_workFile, "q %f %f %f %f %g %g cm BT %s %g Tf %d Tr %g Tz ",
+        fprintf( m_workFile, "q %f %f %f %f %f %f cm BT %s %g Tf %d Tr %g Tz ",
                  ctm_a, ctm_b, ctm_c, ctm_d, ctm_e, ctm_f,
                  fontname, heightFactor, render_mode, wideningFactor * 100 );
 
