@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2018 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2015 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2007-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -312,7 +312,7 @@ void DISPLAY_FOOTPRINTS_FRAME::ReCreateHToolbar()
     UpdateZoomSelectBox();
     m_mainToolBar->AddControl( m_zoomSelectBox );
 
-    // Option to run Zoom automatique on footprint selection changge
+    // Option to run Zoom automatique on footprint selection change
     m_mainToolBar->AddTool( ID_CVPCB_FPVIEWER_AUTOZOOM_TOOL, wxEmptyString,
                             KiScaledBitmap( BITMAPS::zoom_auto_fit_in_page, this ),
                             _( "Automatic Zoom on footprint change" ),
@@ -371,20 +371,20 @@ void DISPLAY_FOOTPRINTS_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 
 WINDOW_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetWindowSettings( APP_SETTINGS_BASE* aCfg )
 {
-    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>( "cvpcb" );
     return &cfg->m_FootprintViewer;
 }
 
 
 PCB_VIEWERS_SETTINGS_BASE* DISPLAY_FOOTPRINTS_FRAME::GetViewerSettingsBase() const
 {
-    return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+    return Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>( "cvpcb" );
 }
 
 
 MAGNETIC_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetMagneticItemsSettings()
 {
-    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>();
+    CVPCB_SETTINGS* cfg = Pgm().GetSettingsManager().GetAppSettings<CVPCB_SETTINGS>( "cvpcb" );
     return &cfg->m_FootprintViewerMagneticSettings;
 }
 
@@ -418,7 +418,8 @@ FOOTPRINT* DISPLAY_FOOTPRINTS_FRAME::GetFootprint( const wxString& aFootprintNam
     // See if the library requested is in the library table
     if( !fpTable->HasLibrary( libNickname ) )
     {
-        aReporter.Report( wxString::Format( _( "Library '%s' is not in the footprint library table." ),
+        aReporter.Report( wxString::Format( _( "Library '%s' is not in the footprint library "
+                                               "table." ),
                                             libNickname ),
                           RPT_SEVERITY_ERROR );
         return nullptr;
@@ -441,7 +442,7 @@ FOOTPRINT* DISPLAY_FOOTPRINTS_FRAME::GetFootprint( const wxString& aFootprintNam
     }
     catch( const IO_ERROR& ioe )
     {
-        DisplayError( this, ioe.What() );
+        DisplayErrorMessage( this, _( "Error loading footprint" ), ioe.What() );
         return nullptr;
     }
 
@@ -525,6 +526,7 @@ void DISPLAY_FOOTPRINTS_FRAME::InitDisplay()
         }
 
         GetBoard()->Add( footprint );
+        GetCanvas()->GetView()->Update( footprint );
         m_currentFootprint = footprintName;
         m_currentComp = comp;
         setFPWatcher( footprint );
@@ -579,10 +581,10 @@ void DISPLAY_FOOTPRINTS_FRAME::UpdateMsgPanel()
 
 COLOR_SETTINGS* DISPLAY_FOOTPRINTS_FRAME::GetColorSettings( bool aForceRefresh ) const
 {
-    auto* settings = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>();
+    auto* cfg = Pgm().GetSettingsManager().GetAppSettings<FOOTPRINT_EDITOR_SETTINGS>( "fpedit" );
 
-    if( settings )
-        return Pgm().GetSettingsManager().GetColorSettings( settings->m_ColorTheme );
+    if( cfg )
+        return Pgm().GetSettingsManager().GetColorSettings( cfg->m_ColorTheme );
     else
         return Pgm().GetSettingsManager().GetColorSettings();
 }
