@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Thomas Pointhuber <thomas.pointhuber@gmx.at>
- * Copyright (C) 2021-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 #define _SCH_IO_ALTIUM_H_
 
 #include <memory>
+#include <sch_sheet_path.h>
 #include <vector>
 #include <sch_io/sch_io.h>
 #include <sch_io/sch_io_mgr.h>
@@ -34,7 +35,7 @@
 
 #include "altium_parser_sch.h"
 
-
+class SYMBOL;
 class SCH_SYMBOL;
 class SCH_SHEET;
 class TITLE_BLOCK;
@@ -76,7 +77,7 @@ public:
 
     SCH_SHEET* LoadSchematicFile( const wxString& aFileName, SCHEMATIC* aSchematic,
                                   SCH_SHEET*             aAppendToMe = nullptr,
-                                  const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                                  const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     // unimplemented functions. Will trigger a not_implemented IO error.
     //void SaveLibrary( const wxString& aFileName, const PROPERTIES* aProperties = NULL ) override;
@@ -87,15 +88,15 @@ public:
 
     void EnumerateSymbolLib( wxArrayString&         aSymbolNameList,
                              const wxString&        aLibraryPath,
-                             const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                             const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     void EnumerateSymbolLib( std::vector<LIB_SYMBOL*>& aSymbolList,
                              const wxString&           aLibraryPath,
-                             const STRING_UTF8_MAP*    aProperties = nullptr ) override;
+                             const std::map<std::string, UTF8>*    aProperties = nullptr ) override;
 
     LIB_SYMBOL* LoadSymbol( const wxString&        aLibraryPath,
                             const wxString&        aAliasName,
-                            const STRING_UTF8_MAP* aProperties = nullptr ) override;
+                            const std::map<std::string, UTF8>* aProperties = nullptr ) override;
 
     //void SaveSymbol( const wxString& aLibraryPath, const LIB_SYMBOL* aSymbol,
     //                 const PROPERTIES* aProperties = NULL ) override;
@@ -152,6 +153,7 @@ private:
     void ParseHarnessType( const std::map<wxString, wxString>& aProperties );
     void ParseHarnessPort( const ASCH_PORT& aElem );
     void ParseHyperlink( const std::map<wxString, wxString>& aProperties, std::vector<LIB_SYMBOL*>& aSymbol  = nullsym);
+    void ParsePieChart( const std::map<wxString, wxString>& aProperties, std::vector<LIB_SYMBOL*>& aSymbol  = nullsym);
     void ParseRectangle( const std::map<wxString, wxString>& aProperties, std::vector<LIB_SYMBOL*>& aSymbol  = nullsym);
     void ParseSheetSymbol( int aIndex, const std::map<wxString, wxString>& aProperties );
     void ParseSheetEntry( const std::map<wxString, wxString>& aProperties );
@@ -178,6 +180,11 @@ private:
     std::map<wxString,LIB_SYMBOL*> ParseLibFile( const ALTIUM_COMPOUND_FILE& aAltiumSchFile );
     std::vector<LIB_SYMBOL*> ParseLibComponent( const std::map<wxString, wxString>& aProperties );
 
+    void doEnumerateSymbolLib( const wxString& aLibraryPath, const std::map<std::string, UTF8>* aProperties,
+                               std::function<void(const wxString&, LIB_SYMBOL*)> aInserter );
+
+    void fixupSymbolPinNameNumbers( SYMBOL* aSymbol );
+
 private:
     SCH_SHEET* m_rootSheet;      // The root sheet of the schematic being loaded..
     SCH_SHEET_PATH m_sheetPath;
@@ -186,7 +193,7 @@ private:
     bool       m_isIntLib;       // Flag to indicate Integrated Library
 
     IO_RELEASER<SCH_IO>               m_pi;                // Plugin to create KiCad symbol library.
-    std::unique_ptr<STRING_UTF8_MAP>  m_properties;        // Library plugin properties.
+    std::unique_ptr<std::map<std::string, UTF8>>  m_properties;        // Library plugin properties.
 
     std::unique_ptr<TITLE_BLOCK>    m_currentTitleBlock; // Will be assigned at the end of parsing
                                                          // a sheet
@@ -214,7 +221,7 @@ private:
     int m_harnessEntryParent; // used to identify harness connector for harness entry element
 
     // Symbol caching
-    void ensureLoadedLibrary( const wxString& aLibraryPath, const STRING_UTF8_MAP* aProperties );
+    void ensureLoadedLibrary( const wxString& aLibraryPath, const std::map<std::string, UTF8>* aProperties );
     long long getLibraryTimestamp( const wxString& aLibraryPath ) const;
 
     static bool isBinaryFile( const wxString& aFileName );
