@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Roberto Fernandez Bautista <roberto.fer.bau@gmail.com>
- * Copyright (C) 2020-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -26,11 +26,11 @@
 #include <lib_symbol.h>
 #include <progress_reporter.h>
 #include <project_sch.h>
-#include <string_utf8_map.h>
 #include <sch_screen.h>
 #include <sch_sheet.h>
 #include <schematic.h>
 #include <sch_io/kicad_sexpr/sch_io_kicad_sexpr.h>
+#include <symbol_lib_table.h>
 #include <wildcards_and_files_ext.h>
 #include <wx_filename.h>
 #include <wx/dir.h>
@@ -65,7 +65,7 @@ int SCH_IO_CADSTAR_ARCHIVE::GetModifyHash() const
 SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFileName,
                                                       SCHEMATIC*             aSchematic,
                                                       SCH_SHEET*             aAppendToMe,
-                                                      const STRING_UTF8_MAP* aProperties )
+                                                      const std::map<std::string, UTF8>* aProperties )
 {
     wxCHECK( !aFileName.IsEmpty() && aSchematic, nullptr );
 
@@ -92,6 +92,9 @@ SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFi
         SCH_SCREEN* screen = new SCH_SCREEN( aSchematic );
         screen->SetFileName( aFileName );
         rootSheet->SetScreen( screen );
+
+        // Virtual root sheet UUID must be the same as the schematic file UUID.
+        const_cast<KIID&>( rootSheet->m_Uuid ) = screen->GetUuid();
     }
 
     CADSTAR_SCH_ARCHIVE_LOADER csaLoader( aFileName, m_reporter, m_progressReporter );
@@ -102,19 +105,8 @@ SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFi
 
     wxCHECK_MSG( libTable, nullptr, "Could not load symbol lib table." );
 
-    // Lets come up with a nice library name
-    wxString libName = aSchematic->Prj().GetProjectName();
-
-    if( libName.IsEmpty() )
-    {
-        wxFileName fn( rootSheet->GetFileName() );
-        libName = fn.GetName();
-    }
-
-    if( libName.IsEmpty() )
-        libName = "noname";
-
-    libName = LIB_ID::FixIllegalChars( libName, true ).wx_str();
+    wxFileName prj_fn = aSchematic->Prj().GetProjectFullName();
+    wxString libName = CADSTAR_SCH_ARCHIVE_LOADER::CreateLibName( prj_fn, nullptr );
 
     wxFileName libFileName( aSchematic->Prj().GetProjectPath(), libName,
                             FILEEXT::KiCadSymbolLibFileExtension );
@@ -132,22 +124,22 @@ SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFi
                 new SYMBOL_LIB_TABLE_ROW( libName, libTableUri, wxString( "KiCad" ) ) );
 
         // Save project symbol library table.
-        wxFileName fn( aSchematic->Prj().GetProjectPath(),
+        wxFileName libtab_fn( aSchematic->Prj().GetProjectPath(),
                        SYMBOL_LIB_TABLE::GetSymbolLibTableFileName() );
 
         // So output formatter goes out of scope and closes the file before reloading.
         {
-            FILE_OUTPUTFORMATTER formatter( fn.GetFullPath() );
+            FILE_OUTPUTFORMATTER formatter( libtab_fn.GetFullPath() );
             libTable->Format( &formatter, 0 );
         }
 
         // Relaod the symbol library table.
-        aSchematic->Prj().SetElem( PROJECT::ELEM_SYMBOL_LIB_TABLE, NULL );
+        aSchematic->Prj().SetElem( PROJECT::ELEM::SYMBOL_LIB_TABLE, NULL );
         PROJECT_SCH::SchSymbolLibTable( &aSchematic->Prj() );
     }
 
     // set properties to prevent save file on every symbol save
-    STRING_UTF8_MAP properties;
+    std::map<std::string, UTF8> properties;
     properties.emplace( SCH_IO_KICAD_SEXPR::PropBuffering, "" );
 
     for( LIB_SYMBOL* const& symbol : csaLoader.GetLoadedSymbols() )
@@ -156,7 +148,7 @@ SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFi
     sch_plugin->SaveLibrary( libFileName.GetFullPath() );
 
     // Link up all symbols in the design to the newly created library
-    for( SCH_SHEET_PATH& sheet : aSchematic->GetSheets() )
+    for( SCH_SHEET_PATH& sheet : aSchematic->Hierarchy() )
     {
         for( SCH_ITEM* item : sheet.LastScreen()->Items().OfType( SCH_SYMBOL_T ) )
         {
@@ -180,7 +172,7 @@ SCH_SHEET* SCH_IO_CADSTAR_ARCHIVE::LoadSchematicFile( const wxString&        aFi
 
 void SCH_IO_CADSTAR_ARCHIVE::EnumerateSymbolLib( wxArrayString&         aSymbolNameList,
                                                  const wxString&        aLibraryPath,
-                                                 const STRING_UTF8_MAP* aProperties )
+                                                 const std::map<std::string, UTF8>* aProperties )
 {
     ensureLoadedLibrary( aLibraryPath, aProperties );
 
@@ -191,7 +183,7 @@ void SCH_IO_CADSTAR_ARCHIVE::EnumerateSymbolLib( wxArrayString&         aSymbolN
 
 void SCH_IO_CADSTAR_ARCHIVE::EnumerateSymbolLib( std::vector<LIB_SYMBOL*>& aSymbolList,
                                                  const wxString&           aLibraryPath,
-                                                 const STRING_UTF8_MAP* aProperties )
+                                                 const std::map<std::string, UTF8>* aProperties )
 {
     ensureLoadedLibrary( aLibraryPath, aProperties );
 
@@ -202,7 +194,7 @@ void SCH_IO_CADSTAR_ARCHIVE::EnumerateSymbolLib( std::vector<LIB_SYMBOL*>& aSymb
 
 LIB_SYMBOL* SCH_IO_CADSTAR_ARCHIVE::LoadSymbol( const wxString&        aLibraryPath,
                                                 const wxString&        aAliasName,
-                                                const STRING_UTF8_MAP* aProperties )
+                                                const std::map<std::string, UTF8>* aProperties )
 {
     ensureLoadedLibrary( aLibraryPath, aProperties );
 
@@ -219,10 +211,10 @@ void SCH_IO_CADSTAR_ARCHIVE::GetAvailableSymbolFields( std::vector<wxString>& aN
 
     for( auto& [libnameStr, libSymbol] : m_libCache )
     {
-        std::vector<LIB_FIELD*> fields;
+        std::vector<SCH_FIELD*> fields;
         libSymbol->GetFields( fields );
 
-        for( LIB_FIELD* field : fields )
+        for( SCH_FIELD* field : fields )
         {
             if( field->IsMandatory() )
                 continue;
@@ -235,7 +227,7 @@ void SCH_IO_CADSTAR_ARCHIVE::GetAvailableSymbolFields( std::vector<wxString>& aN
 }
 
 
-void SCH_IO_CADSTAR_ARCHIVE::GetLibraryOptions( STRING_UTF8_MAP* aListToAppendTo ) const
+void SCH_IO_CADSTAR_ARCHIVE::GetLibraryOptions( std::map<std::string, UTF8>* aListToAppendTo ) const
 {
     ( *aListToAppendTo )["csa"] =
             UTF8( _( "Path to the CADSTAR schematic archive (*.csa) file related to this CADSTAR "
@@ -250,7 +242,7 @@ void SCH_IO_CADSTAR_ARCHIVE::GetLibraryOptions( STRING_UTF8_MAP* aListToAppendTo
 
 
 void SCH_IO_CADSTAR_ARCHIVE::ensureLoadedLibrary( const wxString& aLibraryPath,
-                                                  const STRING_UTF8_MAP* aProperties )
+                                                  const std::map<std::string, UTF8>* aProperties )
 {
     wxFileName csafn;
     wxString   fplibname = "cadstarpcblib";
@@ -258,7 +250,7 @@ void SCH_IO_CADSTAR_ARCHIVE::ensureLoadedLibrary( const wxString& aLibraryPath,
     // Suppress font substitution warnings
     fontconfig::FONTCONFIG::SetReporter( nullptr );
 
-    if( aProperties && aProperties->count( "csa" ) )
+    if( aProperties && aProperties->contains( "csa" ) )
     {
         csafn = wxFileName( aProperties->at( "csa" ) );
 
@@ -297,9 +289,9 @@ void SCH_IO_CADSTAR_ARCHIVE::ensureLoadedLibrary( const wxString& aLibraryPath,
         }
     }
 
-    if( aProperties && aProperties->count( "fplib" ) )
+    if( aProperties && aProperties->contains( "fplib" ) )
     {
-        fplibname = aProperties->at( "fplib" ).wx_str();
+        fplibname = wxString::FromUTF8( aProperties->at( "fplib" ) );
     }
 
     // Get timestamp
