@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2022-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,10 +20,10 @@
 #include <magic_enum.hpp>
 #include <wx/log.h>
 
-#include <lib_shape.h>
-#include <lib_field.h>
-#include <lib_pin.h>
-#include <lib_text.h>
+#include <lib_symbol.h>
+#include <sch_shape.h>
+#include <sch_pin.h>
+#include <sch_text.h>
 #include <macros.h>
 #include <richio.h>
 #include <string_utils.h>
@@ -56,6 +56,9 @@
     ( LIB_VERSION( major, minor ) <= LIB_VERSION( 2, 4 ) )
 
 
+const int fill_tab[3] = { 'N', 'F', 'f' };
+
+
 SCH_IO_KICAD_LEGACY_LIB_CACHE::SCH_IO_KICAD_LEGACY_LIB_CACHE( const wxString& aFullPathAndFileName ) :
     SCH_IO_LIB_CACHE( aFullPathAndFileName )
 {
@@ -212,11 +215,18 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadDocs()
         LIB_SYMBOL_MAP::iterator it = m_symbols.find( aliasName );
 
         if( it == m_symbols.end() )
+        {
             wxLogWarning( "Symbol '%s' not found in library:\n\n"
-                          "'%s'\n\nat line %d offset %d", aliasName, fn.GetFullPath(),
-                          reader.LineNumber(), (int) (line - reader.Line() ) );
+                          "'%s'\n\nat line %d offset %d",
+                          aliasName,
+                          fn.GetFullPath(),
+                          reader.LineNumber(),
+                          (int) (line - reader.Line() ) );
+        }
         else
+        {
             symbol = it->second;
+        }
 
         // Read the current alias associated doc.
         // if the alias does not exist, just skip the description
@@ -394,7 +404,7 @@ LIB_SYMBOL* SCH_IO_KICAD_LEGACY_LIB_CACHE::LoadPart( LINE_READER& aReader, int a
     // Don't set the library alias, this is determined by the symbol library table.
     symbol->SetLibId( LIB_ID( wxEmptyString, symbol->GetName() ) );
 
-    LIB_FIELD& reference = symbol->GetReferenceField();
+    SCH_FIELD& reference = symbol->GetReferenceField();
 
     if( prefix == "~" )
     {
@@ -496,20 +506,20 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadAliases( std::unique_ptr<LIB_SYMBOL>& aS
             LIB_SYMBOL* newSymbol = new LIB_SYMBOL( newAliasName );
 
             // Inherit the parent mandatory field attributes.
-            for( int id = 0; id < MANDATORY_FIELDS; ++id )
+            for( int fieldId : MANDATORY_FIELDS )
             {
-                LIB_FIELD* field = newSymbol->GetFieldById( id );
+                SCH_FIELD* field = newSymbol->GetFieldById( fieldId );
 
-                // the MANDATORY_FIELDS are exactly that in RAM.
+                // the MANDATORY_FIELD_COUNT are exactly that in RAM.
                 wxASSERT( field );
 
-                LIB_FIELD* parentField = aSymbol->GetFieldById( id );
+                SCH_FIELD* parentField = aSymbol->GetFieldById( fieldId );
 
                 wxASSERT( parentField );
 
                 *field = *parentField;
 
-                if( id == VALUE_FIELD )
+                if( fieldId == VALUE_FIELD )
                     field->SetText( newAliasName );
 
                 field->SetParent( newSymbol );
@@ -537,7 +547,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
     if( sscanf( line + 1, "%d", &id ) != 1 || id < 0 )
         SCH_PARSE_ERROR( "invalid field ID", aReader, line + 1 );
 
-    LIB_FIELD* field;
+    SCH_FIELD* field;
 
     // Description was not mandatory until v8.0, so any fields with an index
     // past this point should be user fields
@@ -553,7 +563,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
     else
     {
         id = aSymbol->GetNextAvailableFieldId();
-        field = new LIB_FIELD( aSymbol.get(), id );
+        field = new SCH_FIELD( aSymbol.get(), id );
         aSymbol->AddDrawItem( field, false );
     }
 
@@ -581,7 +591,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
     VECTOR2I pos;
 
     pos.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pos.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pos.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     field->SetPosition( pos );
 
     VECTOR2I textSize;
@@ -647,7 +657,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
             wxChar attr_2 = attributes[2];
 
             if( attr_1 == 'I' )        // Italic
-                field->SetItalic( true );
+                field->SetItalicFlag( true );
             else if( attr_1 != 'N' )   // No italics is default, check for error.
                 SCH_PARSE_ERROR( "invalid field text italic parameter", aReader, line );
 
@@ -659,12 +669,12 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
     }
 
     // Fields in RAM must always have names.
-    if( id >= 0 && id < MANDATORY_FIELDS )
+    if( field->IsMandatory() )
     {
         // Fields in RAM must always have names, because we are trying to get
         // less dependent on field ids and more dependent on names.
         // Plus assumptions are made in the field editors.
-        field->m_name = GetCanonicalFieldName( id );
+        field->SetName( GetCanonicalFieldName( id ) );
 
         // Ensure the VALUE field = the symbol name (can be not the case
         // with malformed libraries: edited by hand, or converted from other tools)
@@ -686,7 +696,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadField( std::unique_ptr<LIB_SYMBOL>& aSym
         while( aSymbol->FindField( candidateFieldName ) != nullptr )
             candidateFieldName = wxString::Format( "%s_%d", fieldName, ++suffix );
 
-        field->m_name = candidateFieldName;
+        field->SetName( candidateFieldName );
     }
 }
 
@@ -713,20 +723,19 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadDrawEntries( std::unique_ptr<LIB_SYMBOL>
         switch( line[0] )
         {
         case 'A':    // Arc
-            aSymbol->AddDrawItem( loadArc( aSymbol, aReader ), false );
+            aSymbol->AddDrawItem( loadArc( aReader ), false );
             break;
 
         case 'C':    // Circle
-            aSymbol->AddDrawItem( loadCircle( aSymbol, aReader ), false );
+            aSymbol->AddDrawItem( loadCircle( aReader ), false );
             break;
 
         case 'T':    // Text
-            aSymbol->AddDrawItem( loadText( aSymbol, aReader, aMajorVersion, aMinorVersion ),
-                                  false );
+            aSymbol->AddDrawItem( loadText( aReader, aMajorVersion, aMinorVersion ), false );
             break;
 
         case 'S':    // Square
-            aSymbol->AddDrawItem( loadRect( aSymbol, aReader ), false );
+            aSymbol->AddDrawItem( loadRect( aReader ), false );
             break;
 
         case 'X':    // Pin Description
@@ -734,11 +743,11 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::loadDrawEntries( std::unique_ptr<LIB_SYMBOL>
             break;
 
         case 'P':    // Polyline
-            aSymbol->AddDrawItem( loadPolyLine( aSymbol, aReader ), false );
+            aSymbol->AddDrawItem( loadPolyLine( aReader ), false );
             break;
 
         case 'B':    // Bezier Curves
-            aSymbol->AddDrawItem( loadBezier( aSymbol, aReader ), false );
+            aSymbol->AddDrawItem( loadBezier( aReader ), false );
             break;
 
         case '#':    // Comment
@@ -835,19 +844,18 @@ static bool MapAnglesV6( int* aAngle1, int* aAngle2 )
 }
 
 
-LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadArc( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                   LINE_READER&                 aReader )
+SCH_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadArc( LINE_READER& aReader )
 {
     const char* line = aReader.Line();
 
     wxCHECK_MSG( strCompare( "A", line, &line ), nullptr, "Invalid arc definition" );
 
-    LIB_SHAPE* arc = new LIB_SHAPE( aSymbol.get(), SHAPE_T::ARC );
+    SCH_SHAPE* arc = new SCH_SHAPE( SHAPE_T::ARC, LAYER_DEVICE );
 
     VECTOR2I center;
 
     center.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    center.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    center.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
 
     arc->SetPosition( center );
 
@@ -877,9 +885,9 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadArc( std::unique_ptr<LIB_SYMBOL>&
         VECTOR2I arcStart, arcEnd;
 
         arcStart.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-        arcStart.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+        arcStart.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
         arcEnd.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-        arcEnd.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+        arcEnd.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
 
         arc->SetStart( arcStart );
         arc->SetEnd( arcEnd );
@@ -891,10 +899,10 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadArc( std::unique_ptr<LIB_SYMBOL>&
         VECTOR2I arcStart( radius, 0 );
         VECTOR2I arcEnd( radius, 0 );
 
-        RotatePoint( arcStart, EDA_ANGLE( -angle1, EDA_ANGLE_T::TENTHS_OF_A_DEGREE_T ) );
+        RotatePoint( arcStart, EDA_ANGLE( angle1, EDA_ANGLE_T::TENTHS_OF_A_DEGREE_T ) );
         arcStart += arc->GetCenter();
         arc->SetStart( arcStart );
-        RotatePoint( arcEnd, EDA_ANGLE( -angle2, EDA_ANGLE_T::TENTHS_OF_A_DEGREE_T ) );
+        RotatePoint( arcEnd, EDA_ANGLE( angle2, EDA_ANGLE_T::TENTHS_OF_A_DEGREE_T ) );
         arcEnd += arc->GetCenter();
         arc->SetEnd( arcEnd );
     }
@@ -906,30 +914,29 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadArc( std::unique_ptr<LIB_SYMBOL>&
      * these points were stored in the file, so we need to mimic the swapping of start/end
      * points rather than using the stored angles in order to properly map edge cases.
      */
-    if( !MapAnglesV6( &angle1, &angle2 ) )
+    if( MapAnglesV6( &angle1, &angle2 ) )
     {
-        VECTOR2I temp = arc->GetStart();
-        arc->SetStart( arc->GetEnd() );
-        arc->SetEnd( temp );
+        VECTOR2I temp = arc->GetEnd();
+        arc->SetEnd( arc->GetStart() );
+        arc->SetStart( temp );
     }
 
     return arc;
 }
 
 
-LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadCircle( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                      LINE_READER&                 aReader )
+SCH_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadCircle( LINE_READER& aReader )
 {
     const char* line = aReader.Line();
 
     wxCHECK_MSG( strCompare( "C", line, &line ), nullptr, "Invalid circle definition" );
 
-    LIB_SHAPE* circle = new LIB_SHAPE( aSymbol.get(), SHAPE_T::CIRCLE );
+    SCH_SHAPE* circle = new SCH_SHAPE( SHAPE_T::CIRCLE, LAYER_DEVICE );
 
     VECTOR2I center;
 
     center.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    center.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    center.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
 
     int radius = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
 
@@ -950,35 +957,29 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadCircle( std::unique_ptr<LIB_SYMBOL
 }
 
 
-LIB_TEXT* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadText( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                   LINE_READER&                 aReader,
-                                                   int                          aMajorVersion,
-                                                   int                          aMinorVersion )
+SCH_TEXT* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadText( LINE_READER& aReader,
+                                                   int aMajorVersion, int aMinorVersion )
 {
     const char* line = aReader.Line();
 
-    wxCHECK_MSG( strCompare( "T", line, &line ), nullptr, "Invalid LIB_TEXT definition" );
-
-    LIB_TEXT* text = new LIB_TEXT( aSymbol.get() );
-    double    angleInTenths = parseInt( aReader, line, &line );
-
-    text->SetTextAngle( EDA_ANGLE( angleInTenths, TENTHS_OF_A_DEGREE_T ) );
+    wxCHECK_MSG( strCompare( "T", line, &line ), nullptr, "Invalid SCH_TEXT definition" );
 
+    double   angleInTenths;
     VECTOR2I center;
+    VECTOR2I size;
+    wxString str;
+    bool     visible;
+    int      unit;
+    int      bodyStyle;
+
+    angleInTenths = parseInt( aReader, line, &line );
 
     center.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    center.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    text->SetPosition( center );
-
-    VECTOR2I size;
-
+    center.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     size.x = size.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    text->SetTextSize( size );
-    text->SetVisible( !parseInt( aReader, line, &line ) );
-    text->SetUnit( parseInt( aReader, line, &line ) );
-    text->SetBodyStyle( parseInt( aReader, line, &line ) );
-
-    wxString str;
+    visible = !parseInt( aReader, line, &line );
+    unit = parseInt( aReader, line, &line );
+    bodyStyle = parseInt( aReader, line, &line );
 
     // If quoted string loading fails, load as not quoted string.
     if( *line == '"' )
@@ -1001,7 +1002,12 @@ LIB_TEXT* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadText( std::unique_ptr<LIB_SYMBOL>&
         str.Replace( "''", "\"" );
     }
 
-    text->SetText( str );
+    SCH_TEXT* text = new SCH_TEXT( center, str, LAYER_DEVICE );
+    text->SetTextAngle( EDA_ANGLE( angleInTenths, TENTHS_OF_A_DEGREE_T ) );
+    text->SetTextSize( size );
+    text->SetVisible( visible );
+    text->SetUnit( unit );
+    text->SetBodyStyle( bodyStyle );
 
     // Here things are murky and not well defined.  At some point it appears the format
     // was changed to add text properties.  However rather than add the token to the end of
@@ -1015,7 +1021,7 @@ LIB_TEXT* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadText( std::unique_ptr<LIB_SYMBOL>&
              && !is_eol( *line ) )
     {
         if( strCompare( "Italic", line, &line ) )
-            text->SetItalic( true );
+            text->SetItalicFlag( true );
         else if( !strCompare( "Normal", line, &line ) )
             SCH_PARSE_ERROR( "invalid text stype, expected 'Normal' or 'Italic'", aReader, line );
 
@@ -1049,25 +1055,24 @@ LIB_TEXT* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadText( std::unique_ptr<LIB_SYMBOL>&
 }
 
 
-LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadRect( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                    LINE_READER&                 aReader )
+SCH_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadRect( LINE_READER& aReader )
 {
     const char* line = aReader.Line();
 
     wxCHECK_MSG( strCompare( "S", line, &line ), nullptr, "Invalid rectangle definition" );
 
-    LIB_SHAPE* rectangle = new LIB_SHAPE( aSymbol.get(), SHAPE_T::RECTANGLE );
+    SCH_SHAPE* rectangle = new SCH_SHAPE( SHAPE_T::RECTANGLE, LAYER_DEVICE );
 
     VECTOR2I pos;
 
     pos.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pos.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pos.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     rectangle->SetPosition( pos );
 
     VECTOR2I end;
 
     end.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    end.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    end.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     rectangle->SetEnd( end );
 
     rectangle->SetUnit( parseInt( aReader, line, &line ) );
@@ -1086,12 +1091,12 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadRect( std::unique_ptr<LIB_SYMBOL>&
 }
 
 
-LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aSymbol,
+SCH_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aSymbol,
                                                  LINE_READER&                 aReader )
 {
     const char* line = aReader.Line();
 
-    wxCHECK_MSG( strCompare( "X", line, &line ), nullptr, "Invalid LIB_PIN definition" );
+    wxCHECK_MSG( strCompare( "X", line, &line ), nullptr, "Invalid SCH_PIN definition" );
 
     wxString name;
     wxString number;
@@ -1135,7 +1140,7 @@ LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aS
     }
 
     pos += tmp.size() + 1;
-    position.y = schIUScale.MilsToIU( (int) num );
+    position.y = -schIUScale.MilsToIU( (int) num );
 
     tmp = tokens.GetNextToken();
 
@@ -1159,11 +1164,16 @@ LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aS
 
     pos += tmp.size() + 1;
 
-    PIN_ORIENTATION orientation = PIN_ORIENTATION::PIN_RIGHT;
-    std::optional<PIN_ORIENTATION> optVal = magic_enum::enum_cast<PIN_ORIENTATION>( tmp[0] );
+    PIN_ORIENTATION orientation;
 
-    if( optVal.has_value() )
-        orientation = optVal.value();
+    switch( static_cast<char>( tmp[0] ) )
+    {
+    case 'U': orientation = PIN_ORIENTATION::PIN_UP; break;
+    case 'D': orientation = PIN_ORIENTATION::PIN_DOWN; break;
+    case 'L': orientation = PIN_ORIENTATION::PIN_LEFT; break;
+    case 'R': /* fall-through */
+    default:  orientation = PIN_ORIENTATION::PIN_RIGHT; break;
+    }
 
     tmp = tokens.GetNextToken();
 
@@ -1239,8 +1249,7 @@ LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aS
                            aReader.LineNumber(), pos );
     }
 
-
-    LIB_PIN* pin = new LIB_PIN( aSymbol.get(),
+    SCH_PIN* pin = new SCH_PIN( aSymbol.get(),
                                 ConvertToNewOverbarNotation( name ),
                                 ConvertToNewOverbarNotation( number ),
                                 orientation,
@@ -1269,7 +1278,7 @@ LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aS
 
         int flags = 0;
 
-        for( int j = tmp.size(); j > 0; )
+        for( int j = (int) tmp.size(); j > 0; )
         {
             switch( tmp[--j].GetValue() )
             {
@@ -1308,14 +1317,13 @@ LIB_PIN* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPin( std::unique_ptr<LIB_SYMBOL>& aS
 }
 
 
-LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPolyLine( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                        LINE_READER&                 aReader )
+SCH_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPolyLine( LINE_READER& aReader )
 {
     const char* line = aReader.Line();
 
     wxCHECK_MSG( strCompare( "P", line, &line ), nullptr, "Invalid poly definition" );
 
-    LIB_SHAPE* polyLine = new LIB_SHAPE( aSymbol.get(), SHAPE_T::POLY );
+    SCH_SHAPE* polyLine = new SCH_SHAPE( SHAPE_T::POLY, LAYER_DEVICE );
 
     int points = parseInt( aReader, line, &line );
     polyLine->SetUnit( parseInt( aReader, line, &line ) );
@@ -1331,7 +1339,7 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPolyLine( std::unique_ptr<LIB_SYMB
     for( int i = 0; i < points; i++ )
     {
         pt.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-        pt.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+        pt.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
         polyLine->AddPoint( pt );
     }
 
@@ -1342,8 +1350,7 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadPolyLine( std::unique_ptr<LIB_SYMB
 }
 
 
-LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadBezier( std::unique_ptr<LIB_SYMBOL>& aSymbol,
-                                                      LINE_READER&                 aReader )
+SCH_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadBezier( LINE_READER& aReader )
 {
     const char* line = aReader.Line();
 
@@ -1353,7 +1360,7 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadBezier( std::unique_ptr<LIB_SYMBOL
 
     wxCHECK_MSG( points == 4, NULL, "Invalid Bezier curve definition" );
 
-    LIB_SHAPE* bezier = new LIB_SHAPE( aSymbol.get(), SHAPE_T::BEZIER );
+    SCH_SHAPE* bezier = new SCH_SHAPE( SHAPE_T::BEZIER, LAYER_DEVICE );
 
     bezier->SetUnit( parseInt( aReader, line, &line ) );
     bezier->SetBodyStyle( parseInt( aReader, line, &line ) );
@@ -1366,22 +1373,22 @@ LIB_SHAPE* SCH_IO_KICAD_LEGACY_LIB_CACHE::loadBezier( std::unique_ptr<LIB_SYMBOL
     VECTOR2I pt;
 
     pt.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pt.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pt.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     bezier->SetStart( pt );
 
     pt.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pt.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pt.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     bezier->SetBezierC1( pt );
 
     pt.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pt.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pt.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     bezier->SetBezierC2( pt );
 
     pt.x = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
-    pt.y = schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
+    pt.y = -schIUScale.MilsToIU( parseInt( aReader, line, &line ) );
     bezier->SetEnd( pt );
 
-    bezier->RebuildBezierToSegmentsPointsList( bezier->GetWidth() );
+    bezier->RebuildBezierToSegmentsPointsList( bezier->GetWidth() / 2 );
 
     if( *line != 0 )
         bezier->SetFillMode( parseFillMode( aReader, line, &line ) );
@@ -1481,7 +1488,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
         }
     }
 
-    LIB_FIELD&  value = aSymbol->GetValueField();
+    SCH_FIELD&  value = aSymbol->GetValueField();
 
     // First line: it s a comment (symbol name for readers)
     aFormatter.Print( 0, "#\n# %s\n#\n", TO_UTF8( value.GetText() ) );
@@ -1490,7 +1497,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
     aFormatter.Print( 0, "DEF" );
     aFormatter.Print( 0, " %s", TO_UTF8( value.GetText() ) );
 
-    LIB_FIELD& reference = aSymbol->GetReferenceField();
+    SCH_FIELD& reference = aSymbol->GetReferenceField();
 
     if( !reference.GetText().IsEmpty() )
         aFormatter.Print( 0, " %s", TO_UTF8( reference.GetText() ) );
@@ -1499,8 +1506,8 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
 
     aFormatter.Print( 0, " %d %d %c %c %d %c %c\n",
                       0, schIUScale.IUToMils( aSymbol->GetPinNameOffset() ),
-                      aSymbol->ShowPinNumbers() ? 'Y' : 'N',
-                      aSymbol->ShowPinNames() ? 'Y' : 'N',
+                      aSymbol->GetShowPinNumbers() ? 'Y' : 'N',
+                      aSymbol->GetShowPinNames() ? 'Y' : 'N',
                       aSymbol->GetUnitCount(), aSymbol->UnitsLocked() ? 'L' : 'F',
                       aSymbol->IsPower() ? 'P' : 'N' );
 
@@ -1518,29 +1525,35 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
         aFormatter.Print( 0, "Ti %d/%d/%d %d:%d:%d\n", year, mon, day, hour, min, sec );
     }
 
-    std::vector<LIB_FIELD*> fields;
+    std::vector<SCH_FIELD*> fields;
     aSymbol->GetFields( fields );
 
     // Mandatory fields:
     // may have their own save policy so there is a separate loop for them.
     // Empty fields are saved, because the user may have set visibility,
     // size and orientation
-    for( int i = 0; i < MANDATORY_FIELDS; ++i )
-        saveField( fields[i], aFormatter );
+    for( SCH_FIELD* field : fields )
+    {
+        if( field->IsMandatory() )
+            saveField( field, aFormatter );
+    }
 
     // User defined fields:
     // may have their own save policy so there is a separate loop for them.
-    int fieldId = MANDATORY_FIELDS;     // really wish this would go away.
+    int fieldId = MANDATORY_FIELD_COUNT;     // really wish this would go away.
 
-    for( unsigned i = MANDATORY_FIELDS; i < fields.size(); ++i )
+    for( SCH_FIELD* field : fields )
     {
+        if( field->IsMandatory() )
+            continue;
+
         // There is no need to save empty fields, i.e. no reason to preserve field
         // names now that fields names come in dynamically through the template
         // fieldnames.
-        if( !fields[i]->GetText().IsEmpty() )
+        if( !field->GetText().IsEmpty() )
         {
-            fields[i]->SetId( fieldId++ );
-            saveField( fields[i], aFormatter );
+            field->SetId( fieldId++ );
+            saveField( field, aFormatter );
         }
     }
 
@@ -1576,28 +1589,37 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
 
         aFormatter.Print( 0, "DRAW\n" );
 
-        for( LIB_ITEM& item : aSymbol->GetDrawItems() )
+        for( SCH_ITEM& item : aSymbol->GetDrawItems() )
         {
             switch( item.Type() )
             {
-            default:
-            case LIB_FIELD_T:     /* Fields have already been saved above. */  break;
-            case LIB_PIN_T:       savePin( (LIB_PIN* ) &item, aFormatter );    break;
-            case LIB_TEXT_T:      saveText( ( LIB_TEXT* ) &item, aFormatter ); break;
-            case LIB_SHAPE_T:
+            case SCH_PIN_T:
+                savePin( static_cast<SCH_PIN*>( &item ), aFormatter );
+
+                break;
+            case SCH_TEXT_T:
+                saveText( static_cast<SCH_TEXT*>( &item ), aFormatter );
+                break;
+
+            case SCH_SHAPE_T:
             {
-                LIB_SHAPE& shape = static_cast<LIB_SHAPE&>( item );
+                SCH_SHAPE& shape = static_cast<SCH_SHAPE&>( item );
 
                 switch( shape.GetShape() )
                 {
-                case SHAPE_T::ARC:    saveArc( &shape, aFormatter );           break;
-                case SHAPE_T::BEZIER: saveBezier( &shape, aFormatter );        break;
-                case SHAPE_T::CIRCLE: saveCircle( &shape, aFormatter );        break;
-                case SHAPE_T::POLY:   savePolyLine( &shape, aFormatter );      break;
-                case SHAPE_T::RECTANGLE:   saveRectangle( &shape, aFormatter );     break;
+                case SHAPE_T::ARC:       saveArc( &shape, aFormatter );        break;
+                case SHAPE_T::BEZIER:    saveBezier( &shape, aFormatter );     break;
+                case SHAPE_T::CIRCLE:    saveCircle( &shape, aFormatter );     break;
+                case SHAPE_T::POLY:      savePolyLine( &shape, aFormatter );   break;
+                case SHAPE_T::RECTANGLE: saveRectangle( &shape, aFormatter );  break;
                 default:                                                       break;
                 }
+
+                break;
             }
+
+            default:
+                break;
             }
         }
 
@@ -1608,19 +1630,19 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMA
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveArc( LIB_SHAPE* aArc, OUTPUTFORMATTER& aFormatter )
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveArc( SCH_SHAPE* aArc, OUTPUTFORMATTER& aFormatter )
 {
     wxCHECK_RET( aArc && aArc->GetShape() == SHAPE_T::ARC, "Invalid ARC object." );
 
     EDA_ANGLE startAngle, endAngle;
 
-    aArc->CalcArcAngles( startAngle, endAngle );
+    aArc->CalcArcAngles( endAngle, startAngle );
     startAngle.Normalize180();
     endAngle.Normalize180();
 
     aFormatter.Print( 0, "A %d %d %d %d %d %d %d %d %c %d %d %d %d\n",
                       schIUScale.IUToMils( aArc->GetPosition().x ),
-                      schIUScale.IUToMils( aArc->GetPosition().y ),
+                      schIUScale.IUToMils( -aArc->GetPosition().y ),
                       schIUScale.IUToMils( aArc->GetRadius() ),
                       startAngle.AsTenthsOfADegree(),
                       endAngle.AsTenthsOfADegree(),
@@ -1629,13 +1651,13 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveArc( LIB_SHAPE* aArc, OUTPUTFORMATTER& a
                       schIUScale.IUToMils( aArc->GetWidth() ),
                       fill_tab[ static_cast<int>( aArc->GetFillMode() ) - 1 ],
                       schIUScale.IUToMils( aArc->GetStart().x ),
-                      schIUScale.IUToMils( aArc->GetStart().y ),
+                      schIUScale.IUToMils( -aArc->GetStart().y ),
                       schIUScale.IUToMils( aArc->GetEnd().x ),
-                      schIUScale.IUToMils( aArc->GetEnd().y ) );
+                      schIUScale.IUToMils( -aArc->GetEnd().y ) );
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveBezier( LIB_SHAPE* aBezier, OUTPUTFORMATTER& aFormatter )
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveBezier( SCH_SHAPE* aBezier, OUTPUTFORMATTER& aFormatter )
 {
     wxCHECK_RET( aBezier && aBezier->GetShape() == SHAPE_T::BEZIER, "Invalid BEZIER object." );
 
@@ -1646,25 +1668,25 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveBezier( LIB_SHAPE* aBezier, OUTPUTFORMAT
 
     aFormatter.Print( 0, " %d %d %d %d %d %d %d %d",
                       schIUScale.IUToMils( aBezier->GetStart().x ),
-                      schIUScale.IUToMils( aBezier->GetStart().y ),
+                      schIUScale.IUToMils( -aBezier->GetStart().y ),
                       schIUScale.IUToMils( aBezier->GetBezierC1().x ),
-                      schIUScale.IUToMils( aBezier->GetBezierC1().y ),
+                      schIUScale.IUToMils( -aBezier->GetBezierC1().y ),
                       schIUScale.IUToMils( aBezier->GetBezierC2().x ),
-                      schIUScale.IUToMils( aBezier->GetBezierC2().y ),
+                      schIUScale.IUToMils( -aBezier->GetBezierC2().y ),
                       schIUScale.IUToMils( aBezier->GetEnd().x ),
-                      schIUScale.IUToMils( aBezier->GetEnd().y ) );
+                      schIUScale.IUToMils( -aBezier->GetEnd().y ) );
 
     aFormatter.Print( 0, " %c\n", fill_tab[ static_cast<int>( aBezier->GetFillMode() ) - 1 ] );
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveCircle( LIB_SHAPE* aCircle, OUTPUTFORMATTER& aFormatter )
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveCircle( SCH_SHAPE* aCircle, OUTPUTFORMATTER& aFormatter )
 {
     wxCHECK_RET( aCircle && aCircle->GetShape() == SHAPE_T::CIRCLE, "Invalid CIRCLE object." );
 
     aFormatter.Print( 0, "C %d %d %d %d %d %d %c\n",
                       schIUScale.IUToMils( aCircle->GetPosition().x ),
-                      schIUScale.IUToMils( aCircle->GetPosition().y ),
+                      schIUScale.IUToMils( -aCircle->GetPosition().y ),
                       schIUScale.IUToMils( aCircle->GetRadius() ),
                       aCircle->GetUnit(),
                       aCircle->GetBodyStyle(),
@@ -1673,10 +1695,10 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveCircle( LIB_SHAPE* aCircle, OUTPUTFORMAT
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveField( const LIB_FIELD* aField,
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveField( const SCH_FIELD* aField,
                                                OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aField && aField->Type() == LIB_FIELD_T, "Invalid LIB_FIELD object." );
+    wxCHECK_RET( aField && aField->Type() == SCH_FIELD_T, "Invalid SCH_FIELD object." );
 
     int      hjustify, vjustify;
     int      id = aField->GetId();
@@ -1700,7 +1722,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveField( const LIB_FIELD* aField,
                       id,
                       EscapedUTF8( text ).c_str(),       // wraps in quotes
                       schIUScale.IUToMils( aField->GetTextPos().x ),
-                      schIUScale.IUToMils( aField->GetTextPos().y ),
+                      schIUScale.IUToMils( -aField->GetTextPos().y ),
                       schIUScale.IUToMils( aField->GetTextWidth() ),
                       aField->GetTextAngle().IsHorizontal() ? 'H' : 'V',
                       aField->IsVisible() ? 'V' : 'I',
@@ -1708,23 +1730,22 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveField( const LIB_FIELD* aField,
                       aField->IsItalic() ? 'I' : 'N',
                       aField->IsBold() ? 'B' : 'N' );
 
-    /* Save field name, if necessary
-     * Field name is saved only if it is not the default name.
-     * Just because default name depends on the language and can change from
-     * a country to another
-     */
-    wxString defName = TEMPLATE_FIELDNAME::GetDefaultFieldName( id );
-
-    if( id >= MANDATORY_FIELDS && !aField->m_name.IsEmpty() && aField->m_name != defName )
-        aFormatter.Print( 0, " %s", EscapedUTF8( aField->m_name ).c_str() );
+    // Translated names were stored in legacy files, so it's important not to save the
+    // default names as they weren't yet canonical.
+    if( !aField->IsMandatory()
+            && !aField->GetName().IsEmpty()
+            && aField->GetName() != GetUserFieldName( id, !DO_TRANSLATE ) )
+    {
+        aFormatter.Print( 0, " %s", EscapedUTF8( aField->GetName() ).c_str() );
+    }
 
     aFormatter.Print( 0, "\n" );
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePin( const LIB_PIN* aPin, OUTPUTFORMATTER& aFormatter )
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePin( const SCH_PIN* aPin, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aPin && aPin->Type() == LIB_PIN_T, "Invalid LIB_PIN object." );
+    wxCHECK_RET( aPin && aPin->Type() == SCH_PIN_T, "Invalid SCH_PIN object." );
 
     int      Etype;
 
@@ -1749,12 +1770,23 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePin( const LIB_PIN* aPin, OUTPUTFORMATTE
     else
         aFormatter.Print( 0, "X ~" );
 
+    int pin_orient = 'L';   // Printed as a char in lib file
+
+    switch( aPin->GetOrientation() )
+    {
+        case PIN_ORIENTATION::PIN_RIGHT: pin_orient = 'R'; break;
+        case PIN_ORIENTATION::PIN_LEFT: pin_orient = 'L'; break;
+        case PIN_ORIENTATION::PIN_UP: pin_orient = 'U'; break;
+        case PIN_ORIENTATION::PIN_DOWN: pin_orient = 'D'; break;
+        case PIN_ORIENTATION::INHERIT: pin_orient = 'L'; break;     // Should not happens
+    }
+
     aFormatter.Print( 0, " %s %d %d %d %c %d %d %d %d %c",
                       aPin->GetNumber().IsEmpty() ? "~" : TO_UTF8( aPin->GetNumber() ),
                       schIUScale.IUToMils( aPin->GetPosition().x ),
-                      schIUScale.IUToMils( aPin->GetPosition().y ),
+                      schIUScale.IUToMils( -aPin->GetPosition().y ),
                       schIUScale.IUToMils( (int) aPin->GetLength() ),
-                      (int) aPin->GetOrientation(),
+                      pin_orient,
                       schIUScale.IUToMils( aPin->GetNumberTextSize() ),
                       schIUScale.IUToMils( aPin->GetNameTextSize() ),
                       aPin->GetUnit(),
@@ -1783,11 +1815,11 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePin( const LIB_PIN* aPin, OUTPUTFORMATTE
 
     aFormatter.Print( 0, "\n" );
 
-    const_cast<LIB_PIN*>( aPin )->ClearFlags( IS_CHANGED );
+    const_cast<SCH_PIN*>( aPin )->ClearFlags( IS_CHANGED );
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePolyLine( LIB_SHAPE* aPolyLine,
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePolyLine( SCH_SHAPE* aPolyLine,
                                                   OUTPUTFORMATTER& aFormatter )
 {
     wxCHECK_RET( aPolyLine && aPolyLine->GetShape() == SHAPE_T::POLY, "Invalid POLY object." );
@@ -1799,13 +1831,13 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::savePolyLine( LIB_SHAPE* aPolyLine,
                       schIUScale.IUToMils( aPolyLine->GetWidth() ) );
 
     for( const VECTOR2I& pt : aPolyLine->GetPolyShape().Outline( 0 ).CPoints() )
-        aFormatter.Print( 0, " %d %d", schIUScale.IUToMils( pt.x ), schIUScale.IUToMils( pt.y ) );
+        aFormatter.Print( 0, " %d %d", schIUScale.IUToMils( pt.x ), -schIUScale.IUToMils( pt.y ) );
 
     aFormatter.Print( 0, " %c\n", fill_tab[ static_cast<int>( aPolyLine->GetFillMode() ) - 1 ] );
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveRectangle( LIB_SHAPE* aRectangle,
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveRectangle( SCH_SHAPE* aRectangle,
                                                    OUTPUTFORMATTER& aFormatter )
 {
     wxCHECK_RET( aRectangle && aRectangle->GetShape() == SHAPE_T::RECTANGLE,
@@ -1813,9 +1845,9 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveRectangle( LIB_SHAPE* aRectangle,
 
     aFormatter.Print( 0, "S %d %d %d %d %d %d %d %c\n",
                       schIUScale.IUToMils( aRectangle->GetPosition().x ),
-                      schIUScale.IUToMils( aRectangle->GetPosition().y ),
+                      schIUScale.IUToMils( -aRectangle->GetPosition().y ),
                       schIUScale.IUToMils( aRectangle->GetEnd().x ),
-                      schIUScale.IUToMils( aRectangle->GetEnd().y ),
+                      schIUScale.IUToMils( -aRectangle->GetEnd().y ),
                       aRectangle->GetUnit(),
                       aRectangle->GetBodyStyle(),
                       schIUScale.IUToMils( aRectangle->GetWidth() ),
@@ -1823,9 +1855,9 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveRectangle( LIB_SHAPE* aRectangle,
 }
 
 
-void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveText( const LIB_TEXT* aText, OUTPUTFORMATTER& aFormatter )
+void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveText( const SCH_TEXT* aText, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aText && aText->Type() == LIB_TEXT_T, "Invalid LIB_TEXT object." );
+    wxCHECK_RET( aText && aText->Type() == SCH_TEXT_T, "Invalid SCH_TEXT object." );
 
     wxString text = aText->GetText();
 
@@ -1839,7 +1871,7 @@ void SCH_IO_KICAD_LEGACY_LIB_CACHE::saveText( const LIB_TEXT* aText, OUTPUTFORMA
     aFormatter.Print( 0, "T %g %d %d %d %d %d %d %s",
                       (double) aText->GetTextAngle().AsTenthsOfADegree(),
                       schIUScale.IUToMils( aText->GetTextPos().x ),
-                      schIUScale.IUToMils( aText->GetTextPos().y ),
+                      schIUScale.IUToMils( -aText->GetTextPos().y ),
                       schIUScale.IUToMils( aText->GetTextWidth() ),
                       !aText->IsVisible(),
                       aText->GetUnit(),
