@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2022-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Wayne Stambaugh <stambaughw@gmail.com>
  *
@@ -22,10 +22,9 @@
 #include <wx/log.h>
 #include <base_units.h>
 #include <build_version.h>
-#include <lib_shape.h>
+#include <sch_shape.h>
 #include <lib_symbol.h>
-#include <lib_text.h>
-#include <lib_textbox.h>
+#include <sch_textbox.h>
 #include <locale_io.h>
 #include <macros.h>
 #include <richio.h>
@@ -34,6 +33,7 @@
 #include "sch_io_kicad_sexpr_parser.h"
 #include <string_utils.h>
 #include <trace_helpers.h>
+#include <io/kicad/kicad_io_utils.h>
 
 
 SCH_IO_KICAD_SEXPR_LIB_CACHE::SCH_IO_KICAD_SEXPR_LIB_CACHE( const wxString& aFullPathAndFileName ) :
@@ -72,7 +72,7 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::Load()
     SCH_IO_KICAD_SEXPR_PARSER parser( &reader );
 
     parser.ParseLib( m_symbols );
-    ++m_modHash;
+    IncrementModifyHash();
 
     // Remember the file modification time of library file when the cache snapshot was made,
     // so that in a networked environment we will reload the cache as needed.
@@ -93,9 +93,10 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::Save( const std::optional<bool>& aOpt )
 
     auto formatter = std::make_unique<PRETTIFIED_FILE_OUTPUTFORMATTER>( fn.GetFullPath() );
 
-    formatter->Print( 0, "(kicad_symbol_lib (version %d) (generator \"kicad_symbol_editor\") "
-                      "(generator_version \"%s\")\n",
-                      SEXPR_SYMBOL_LIB_FILE_VERSION, GetMajorMinorVersion().c_str().AsChar() );
+    formatter->Print( "(kicad_symbol_lib (version %d) (generator \"kicad_symbol_editor\") "
+                      "(generator_version \"%s\")",
+                      SEXPR_SYMBOL_LIB_FILE_VERSION,
+                      GetMajorMinorVersion().c_str().AsChar() );
 
     std::vector<LIB_SYMBOL*> orderedSymbols;
 
@@ -119,9 +120,9 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::Save( const std::optional<bool>& aOpt )
                } );
 
     for( LIB_SYMBOL* symbol : orderedSymbols )
-        SaveSymbol( symbol, *formatter.get(), 1 );
+        SaveSymbol( symbol, *formatter.get() );
 
-    formatter->Print( 0, ")\n" );
+    formatter->Print( ")" );
 
     formatter.reset();
 
@@ -131,7 +132,7 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::Save( const std::optional<bool>& aOpt )
 
 
 void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMATTER& aFormatter,
-                                               int aNestLevel, const wxString& aLibName )
+                                               const wxString& aLibName, bool aIncludeData )
 {
     wxCHECK_RET( aSymbol, "Invalid LIB_SYMBOL pointer." );
 
@@ -139,8 +140,17 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
     wxCHECK2( wxLocale::GetInfo( wxLOCALE_DECIMAL_POINT, wxLOCALE_CAT_NUMBER ) == ".",
               LOCALE_IO toggle );
 
-    int nextFreeFieldId = MANDATORY_FIELDS;
-    std::vector<LIB_FIELD*> fields;
+
+    // If we've requested to embed the fonts in the symbol, do so.
+    // Otherwise, clear the embedded fonts from the symbol.  Embedded
+    // fonts will be used if available
+    if( aSymbol->GetAreFontsEmbedded() )
+        aSymbol->EmbedFonts();
+    else
+        aSymbol->GetEmbeddedFiles()->ClearEmbeddedFonts();
+
+    int nextFreeFieldId = MANDATORY_FIELD_COUNT;
+    std::vector<SCH_FIELD*> fields;
     std::string name = aFormatter.Quotew( aSymbol->GetLibId().GetLibItemName().wx_str() );
     std::string unitName = aSymbol->GetLibId().GetLibItemName();
 
@@ -157,49 +167,48 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
 
     if( aSymbol->IsRoot() )
     {
-        aFormatter.Print( aNestLevel, "(symbol %s", name.c_str() );
+        aFormatter.Print( "(symbol %s", name.c_str() );
 
         if( aSymbol->IsPower() )
-            aFormatter.Print( 0, " (power)" );
+            aFormatter.Print( "(power)" );
 
         // TODO: add uuid token here.
 
         // TODO: add anchor position token here.
 
-        if( !aSymbol->ShowPinNumbers() )
-            aFormatter.Print( 0, " (pin_numbers hide)" );
+        if( !aSymbol->GetShowPinNumbers() )
+            aFormatter.Print( "(pin_numbers (hide yes))" );
 
         if( aSymbol->GetPinNameOffset() != schIUScale.MilsToIU( DEFAULT_PIN_NAME_OFFSET )
-          || !aSymbol->ShowPinNames() )
+          || !aSymbol->GetShowPinNames() )
         {
-            aFormatter.Print( 0, " (pin_names" );
+            aFormatter.Print( "(pin_names" );
 
             if( aSymbol->GetPinNameOffset() != schIUScale.MilsToIU( DEFAULT_PIN_NAME_OFFSET ) )
-                aFormatter.Print( 0, " (offset %s)",
+            {
+                aFormatter.Print( "(offset %s)",
                                   EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                                        aSymbol->GetPinNameOffset() ).c_str() );
+            }
 
-            if( !aSymbol->ShowPinNames() )
-                aFormatter.Print( 0, " hide" );
+            if( !aSymbol->GetShowPinNames() )
+                KICAD_FORMAT::FormatBool( &aFormatter, "hide", true );
 
-            aFormatter.Print( 0, ")" );
+            aFormatter.Print( ")" );
         }
 
-        aFormatter.Print( 0, " (exclude_from_sim %s)",
-                          ( aSymbol->GetExcludedFromSim() ) ? "yes" : "no" );
-        aFormatter.Print( 0, " (in_bom %s)", ( aSymbol->GetExcludedFromBOM() ) ? "no" : "yes" );
-        aFormatter.Print( 0, " (on_board %s)", ( aSymbol->GetExcludedFromBoard() ) ? "no" : "yes" );
+        KICAD_FORMAT::FormatBool( &aFormatter, "exclude_from_sim",  aSymbol->GetExcludedFromSim() );
+        KICAD_FORMAT::FormatBool( &aFormatter, "in_bom", !aSymbol->GetExcludedFromBOM() );
+        KICAD_FORMAT::FormatBool( &aFormatter, "on_board", !aSymbol->GetExcludedFromBoard() );
 
         // TODO: add atomic token here.
 
         // TODO: add required token here."
 
-        aFormatter.Print( 0, "\n" );
-
         aSymbol->GetFields( fields );
 
-        for( LIB_FIELD* field : fields )
-            saveField( field, aFormatter, aNestLevel + 1 );
+        for( SCH_FIELD* field : fields )
+            saveField( field, aFormatter );
 
         nextFreeFieldId = aSymbol->GetNextAvailableFieldId();
 
@@ -208,12 +217,12 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
         // locked flag state.
         if( aSymbol->UnitsLocked() )
         {
-            LIB_FIELD locked( nextFreeFieldId, "ki_locked" );
-            saveField( &locked, aFormatter, aNestLevel + 1 );
+            SCH_FIELD locked( nullptr, nextFreeFieldId, "ki_locked" );
+            saveField( &locked, aFormatter );
             nextFreeFieldId += 1;
         }
 
-        saveDcmInfoAsFields( aSymbol, aFormatter, nextFreeFieldId, aNestLevel );
+        saveDcmInfoAsFields( aSymbol, aFormatter, nextFreeFieldId );
 
         // Save the draw items grouped by units.
         std::vector<LIB_SYMBOL_UNIT> units = aSymbol->GetUnitDrawItems();
@@ -232,7 +241,7 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
             name = aFormatter.Quotes( unitName );
             name.pop_back();    // Remove last char: the quote ending the string.
 
-            aFormatter.Print( aNestLevel + 1, "(symbol %s_%d_%d\"\n",
+            aFormatter.Print( "(symbol %s_%d_%d\"",
                               name.c_str(),
                               unit.m_unit,
                               unit.m_bodyStyle );
@@ -241,26 +250,31 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
             if( aSymbol->HasUnitDisplayName( unit.m_unit ) )
             {
                 name = aSymbol->GetUnitDisplayName( unit.m_unit );
-                aFormatter.Print( aNestLevel + 2, "(unit_name %s)\n",
-                                  aFormatter.Quotes( name ).c_str() );
+                aFormatter.Print( "(unit_name %s)", aFormatter.Quotes( name ).c_str() );
             }
+
             // Enforce item ordering
             auto cmp =
-                    []( const LIB_ITEM* a, const LIB_ITEM* b )
+                    []( const SCH_ITEM* a, const SCH_ITEM* b )
                     {
                         return *a < *b;
                     };
 
-            std::multiset<LIB_ITEM*, decltype( cmp )> save_map( cmp );
+            std::multiset<SCH_ITEM*, decltype( cmp )> save_map( cmp );
 
-            for( LIB_ITEM* item : unit.m_items )
+            for( SCH_ITEM* item : unit.m_items )
                 save_map.insert( item );
 
-            for( LIB_ITEM* item : save_map )
-                saveSymbolDrawItem( item, aFormatter, aNestLevel + 2 );
+            for( SCH_ITEM* item : save_map )
+                saveSymbolDrawItem( item, aFormatter );
 
-            aFormatter.Print( aNestLevel + 1, ")\n" );
+            aFormatter.Print( ")" );
         }
+
+        KICAD_FORMAT::FormatBool( &aFormatter, "embedded_fonts", aSymbol->GetAreFontsEmbedded() );
+
+        if( !aSymbol->EmbeddedFileMap().empty() )
+            aSymbol->WriteEmbeddedFiles( aFormatter, aIncludeData );
     }
     else
     {
@@ -268,36 +282,36 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::SaveSymbol( LIB_SYMBOL* aSymbol, OUTPUTFORMAT
 
         wxASSERT( parent );
 
-        aFormatter.Print( aNestLevel, "(symbol %s (extends %s)\n",
+        aFormatter.Print( "(symbol %s (extends %s)",
                           name.c_str(),
                           aFormatter.Quotew( parent->GetName() ).c_str() );
 
         aSymbol->GetFields( fields );
 
-        for( LIB_FIELD* field : fields )
-            saveField( field, aFormatter, aNestLevel + 1 );
+        for( SCH_FIELD* field : fields )
+            saveField( field, aFormatter );
 
         nextFreeFieldId = aSymbol->GetNextAvailableFieldId();
 
-        saveDcmInfoAsFields( aSymbol, aFormatter, nextFreeFieldId, aNestLevel );
+        saveDcmInfoAsFields( aSymbol, aFormatter, nextFreeFieldId );
     }
 
-    aFormatter.Print( aNestLevel, ")\n" );
+    aFormatter.Print( ")" );
 }
 
 
 void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveDcmInfoAsFields( LIB_SYMBOL* aSymbol,
                                                         OUTPUTFORMATTER& aFormatter,
-                                                        int& aNextFreeFieldId, int aNestLevel )
+                                                        int& aNextFreeFieldId )
 {
     wxCHECK_RET( aSymbol, "Invalid LIB_SYMBOL pointer." );
 
     if( !aSymbol->GetKeyWords().IsEmpty() )
     {
-        LIB_FIELD keywords( aNextFreeFieldId, wxString( "ki_keywords" ) );
+        SCH_FIELD keywords( nullptr, aNextFreeFieldId, wxString( "ki_keywords" ) );
         keywords.SetVisible( false );
         keywords.SetText( aSymbol->GetKeyWords() );
-        saveField( &keywords, aFormatter, aNestLevel + 1 );
+        saveField( &keywords, aFormatter );
         aNextFreeFieldId += 1;
     }
 
@@ -318,25 +332,24 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveDcmInfoAsFields( LIB_SYMBOL* aSymbol,
                 tmp += " " + curr_filter;
         }
 
-        LIB_FIELD description( aNextFreeFieldId, wxString( "ki_fp_filters" ) );
+        SCH_FIELD description( nullptr, aNextFreeFieldId, wxString( "ki_fp_filters" ) );
         description.SetVisible( false );
         description.SetText( tmp );
-        saveField( &description, aFormatter, aNestLevel + 1 );
+        saveField( &description, aFormatter );
         aNextFreeFieldId += 1;
     }
 }
 
 
-void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveSymbolDrawItem( LIB_ITEM* aItem, OUTPUTFORMATTER& aFormatter,
-                                                       int aNestLevel )
+void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveSymbolDrawItem( SCH_ITEM* aItem, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aItem, "Invalid LIB_ITEM pointer." );
+    wxCHECK_RET( aItem, "Invalid SCH_ITEM pointer." );
 
     switch( aItem->Type() )
     {
-    case LIB_SHAPE_T:
+    case SCH_SHAPE_T:
     {
-        LIB_SHAPE*    shape = static_cast<LIB_SHAPE*>( aItem );
+        SCH_SHAPE*    shape = static_cast<SCH_SHAPE*>( aItem );
         STROKE_PARAMS stroke = shape->GetStroke();
         FILL_T        fillMode = shape->GetFillMode();
         COLOR4D       fillColor = shape->GetFillColor();
@@ -345,23 +358,23 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveSymbolDrawItem( LIB_ITEM* aItem, OUTPUTFO
         switch( shape->GetShape() )
         {
         case SHAPE_T::ARC:
-            formatArc( &aFormatter, aNestLevel, shape, isPrivate, stroke, fillMode, fillColor );
+            formatArc( &aFormatter, shape, isPrivate, stroke, fillMode, fillColor, true );
             break;
 
         case SHAPE_T::CIRCLE:
-            formatCircle( &aFormatter, aNestLevel, shape, isPrivate, stroke, fillMode, fillColor );
+            formatCircle( &aFormatter, shape, isPrivate, stroke, fillMode, fillColor, true );
             break;
 
         case SHAPE_T::RECTANGLE:
-            formatRect( &aFormatter, aNestLevel, shape, isPrivate, stroke, fillMode, fillColor );
+            formatRect( &aFormatter, shape, isPrivate, stroke, fillMode, fillColor, true );
             break;
 
         case SHAPE_T::BEZIER:
-            formatBezier(&aFormatter, aNestLevel, shape, isPrivate, stroke, fillMode, fillColor );
+            formatBezier(&aFormatter, shape, isPrivate, stroke, fillMode, fillColor, true );
             break;
 
         case SHAPE_T::POLY:
-            formatPoly( &aFormatter, aNestLevel, shape, isPrivate, stroke, fillMode, fillColor );
+            formatPoly( &aFormatter, shape, isPrivate, stroke, fillMode, fillColor, true );
             break;
 
         default:
@@ -371,16 +384,16 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveSymbolDrawItem( LIB_ITEM* aItem, OUTPUTFO
         break;
     }
 
-    case LIB_PIN_T:
-        savePin( static_cast<LIB_PIN*>( aItem ), aFormatter, aNestLevel );
+    case SCH_PIN_T:
+        savePin( static_cast<SCH_PIN*>( aItem ), aFormatter );
         break;
 
-    case LIB_TEXT_T:
-        saveText( static_cast<LIB_TEXT*>( aItem ), aFormatter, aNestLevel );
+    case SCH_TEXT_T:
+        saveText( static_cast<SCH_TEXT*>( aItem ), aFormatter );
         break;
 
-    case LIB_TEXTBOX_T:
-        saveTextBox( static_cast<LIB_TEXTBOX*>( aItem ), aFormatter, aNestLevel );
+    case SCH_TEXTBOX_T:
+        saveTextBox( static_cast<SCH_TEXTBOX*>( aItem ), aFormatter );
         break;
 
     default:
@@ -389,69 +402,65 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveSymbolDrawItem( LIB_ITEM* aItem, OUTPUTFO
 }
 
 
-void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveField( LIB_FIELD* aField, OUTPUTFORMATTER& aFormatter,
-                                              int aNestLevel )
+void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveField( SCH_FIELD* aField, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aField && aField->Type() == LIB_FIELD_T, "Invalid LIB_FIELD object." );
+    wxCHECK_RET( aField && aField->Type() == SCH_FIELD_T, "Invalid SCH_FIELD object." );
 
     wxString fieldName = aField->GetName();
 
-    if( aField->GetId() >= 0 && aField->GetId() < MANDATORY_FIELDS )
+    if( aField->IsMandatory() )
         fieldName = GetCanonicalFieldName( aField->GetId() );
 
-    aFormatter.Print( aNestLevel, "(property %s %s (at %s %s %g)",
+    aFormatter.Print( "(property %s %s %s (at %s %s %g)",
+                      aField->IsPrivate() ? "private" : "",
                       aFormatter.Quotew( fieldName ).c_str(),
                       aFormatter.Quotew( aField->GetText() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aField->GetPosition().x ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
-                                                           aField->GetPosition().y ).c_str(),
+                                                           -aField->GetPosition().y ).c_str(),
                       aField->GetTextAngle().AsDegrees() );
 
     if( aField->IsNameShown() )
-        aFormatter.Print( 0, " (show_name)" );
+        aFormatter.Print( "(show_name)" );
 
     if( !aField->CanAutoplace() )
-        aFormatter.Print( 0, " (do_not_autoplace)" );
+        aFormatter.Print( "(do_not_autoplace)" );
 
-    aFormatter.Print( 0, "\n" );
-    aField->Format( &aFormatter, aNestLevel, 0 );
-    aFormatter.Print( aNestLevel, ")\n" );
+    aField->Format( &aFormatter, 0 );
+    aFormatter.Print( ")" );
 }
 
 
-void SCH_IO_KICAD_SEXPR_LIB_CACHE::savePin( LIB_PIN* aPin, OUTPUTFORMATTER& aFormatter,
-                                            int aNestLevel )
+void SCH_IO_KICAD_SEXPR_LIB_CACHE::savePin( SCH_PIN* aPin, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aPin && aPin->Type() == LIB_PIN_T, "Invalid LIB_PIN object." );
+    wxCHECK_RET( aPin && aPin->Type() == SCH_PIN_T, "Invalid SCH_PIN object." );
 
     aPin->ClearFlags( IS_CHANGED );
 
-    aFormatter.Print( aNestLevel, "(pin %s %s (at %s %s %s) (length %s)",
+    aFormatter.Print( "(pin %s %s (at %s %s %s) (length %s)",
                       getPinElectricalTypeToken( aPin->GetType() ),
                       getPinShapeToken( aPin->GetShape() ),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aPin->GetPosition().x ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
-                                                           aPin->GetPosition().y ).c_str(),
+                                                           -aPin->GetPosition().y ).c_str(),
                       EDA_UNIT_UTILS::FormatAngle( getPinAngle( aPin->GetOrientation() ) ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aPin->GetLength() ).c_str() );
 
     if( !aPin->IsVisible() )
-        aFormatter.Print( 0, " hide\n" );
-    else
-        aFormatter.Print( 0, "\n" );
+        KICAD_FORMAT::FormatBool( &aFormatter, "hide", true );
 
     // This follows the EDA_TEXT effects formatting for future expansion.
-    aFormatter.Print( aNestLevel + 1, "(name %s (effects (font (size %s %s))))\n",
+    aFormatter.Print( "(name %s (effects (font (size %s %s))))",
                       aFormatter.Quotew( aPin->GetName() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aPin->GetNameTextSize() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aPin->GetNameTextSize() ).c_str() );
 
-    aFormatter.Print( aNestLevel + 1, "(number %s (effects (font (size %s %s))))\n",
+    aFormatter.Print( "(number %s (effects (font (size %s %s))))",
                       aFormatter.Quotew( aPin->GetNumber() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aPin->GetNumberTextSize() ).c_str(),
@@ -459,64 +468,62 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::savePin( LIB_PIN* aPin, OUTPUTFORMATTER& aFor
                                                            aPin->GetNumberTextSize() ).c_str() );
 
 
-    for( const std::pair<const wxString, LIB_PIN::ALT>& alt : aPin->GetAlternates() )
+    for( const std::pair<const wxString, SCH_PIN::ALT>& alt : aPin->GetAlternates() )
     {
-        aFormatter.Print( aNestLevel + 1, "(alternate %s %s %s)\n",
+        aFormatter.Print( "(alternate %s %s %s)",
                           aFormatter.Quotew( alt.second.m_Name ).c_str(),
                           getPinElectricalTypeToken( alt.second.m_Type ),
                           getPinShapeToken( alt.second.m_Shape ) );
     }
 
-    aFormatter.Print( aNestLevel, ")\n" );
+    aFormatter.Print( ")" );
 }
 
 
-void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveText( LIB_TEXT* aText, OUTPUTFORMATTER& aFormatter,
-                                             int aNestLevel )
+void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveText( SCH_TEXT* aText, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aText && aText->Type() == LIB_TEXT_T, "Invalid LIB_TEXT object." );
+    wxCHECK_RET( aText && aText->Type() == SCH_TEXT_T, "Invalid SCH_TEXT object." );
 
-    aFormatter.Print( aNestLevel, "(text%s %s (at %s %s %g)\n",
-                      aText->IsPrivate() ? " private" : "",
+    aFormatter.Print( "(text %s %s (at %s %s %g)",
+                      aText->IsPrivate() ? "private" : "",
                       aFormatter.Quotew( aText->GetText() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
                                                            aText->GetPosition().x ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale,
-                                                           aText->GetPosition().y ).c_str(),
+                                                           -aText->GetPosition().y ).c_str(),
                       (double) aText->GetTextAngle().AsTenthsOfADegree() );
 
-    aText->EDA_TEXT::Format( &aFormatter, aNestLevel, 0 );
-    aFormatter.Print( aNestLevel, ")\n" );
+    aText->EDA_TEXT::Format( &aFormatter, 0 );
+    aFormatter.Print( ")" );
 }
 
 
-void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveTextBox( LIB_TEXTBOX* aTextBox, OUTPUTFORMATTER& aFormatter,
-                                                int aNestLevel )
+void SCH_IO_KICAD_SEXPR_LIB_CACHE::saveTextBox( SCH_TEXTBOX* aTextBox, OUTPUTFORMATTER& aFormatter )
 {
-    wxCHECK_RET( aTextBox && aTextBox->Type() == LIB_TEXTBOX_T, "Invalid LIB_TEXTBOX object." );
+    wxCHECK_RET( aTextBox && aTextBox->Type() == SCH_TEXTBOX_T, "Invalid SCH_TEXTBOX object." );
 
-    aFormatter.Print( aNestLevel, "(text_box%s %s\n",
-                      aTextBox->IsPrivate() ? " private" : "",
+    aFormatter.Print( "(text_box %s %s",
+                      aTextBox->IsPrivate() ? "private" : "",
                       aFormatter.Quotew( aTextBox->GetText() ).c_str() );
 
     VECTOR2I pos = aTextBox->GetStart();
     VECTOR2I size = aTextBox->GetEnd() - pos;
 
-    aFormatter.Print( aNestLevel + 1, "(at %s %s %s) (size %s %s)\n",
+    aFormatter.Print( "(at %s %s %s) (size %s %s) (margins %s %s %s %s)",
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, pos.x ).c_str(),
-                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, pos.y ).c_str(),
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, -pos.y ).c_str(),
                       EDA_UNIT_UTILS::FormatAngle( aTextBox->GetTextAngle() ).c_str(),
                       EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, size.x ).c_str(),
-                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, size.y ).c_str() );
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, -size.y ).c_str(),
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, aTextBox->GetMarginLeft() ).c_str(),
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, aTextBox->GetMarginTop() ).c_str(),
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, aTextBox->GetMarginRight() ).c_str(),
+                      EDA_UNIT_UTILS::FormatInternalUnits( schIUScale, aTextBox->GetMarginBottom() ).c_str() );
 
-    aTextBox->GetStroke().Format( &aFormatter, schIUScale, aNestLevel + 1 );
-    aFormatter.Print( 0, "\n" );
-
-    formatFill( &aFormatter, aNestLevel + 1, aTextBox->GetFillMode(), aTextBox->GetFillColor() );
-    aFormatter.Print( 0, "\n" );
-
-    aTextBox->EDA_TEXT::Format( &aFormatter, aNestLevel, 0 );
-    aFormatter.Print( aNestLevel, ")\n" );
+    aTextBox->GetStroke().Format( &aFormatter, schIUScale );
+    formatFill( &aFormatter, aTextBox->GetFillMode(), aTextBox->GetFillColor() );
+    aTextBox->EDA_TEXT::Format( &aFormatter, 0 );
+    aFormatter.Print( ")" );
 }
 
 
@@ -562,6 +569,6 @@ void SCH_IO_KICAD_SEXPR_LIB_CACHE::DeleteSymbol( const wxString& aSymbolName )
         delete symbol;
     }
 
-    ++m_modHash;
+    IncrementModifyHash();
     m_isModified = true;
 }
