@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2015-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
  *  2 - create a footprint report (pos and footprint descr) (ascii file)
  */
 
+#include <dialog_gen_footprint_position.h>
 #include <confirm.h>
 #include <pcb_edit_frame.h>
 #include <pcbnew_settings.h>
@@ -37,169 +38,200 @@
 #include <kiface_base.h>
 #include <widgets/wx_html_report_panel.h>
 #include <widgets/std_bitmap_button.h>
-#include <dialog_gen_footprint_position_file_base.h>
 #include <exporters/place_file_exporter.h>
 #include "gerber_placefile_writer.h"
+#include <jobs/job_export_pcb_pos.h>
 
 #include <wx/dirdlg.h>
+#include <wx/msgdlg.h>
 
 
-/**
- * The dialog to create footprint position files and choose options (one or 2 files, units
- * and force all SMD footprints in list)
- */
-class DIALOG_GEN_FOOTPRINT_POSITION : public DIALOG_GEN_FOOTPRINT_POSITION_BASE
+DIALOG_GEN_FOOTPRINT_POSITION::DIALOG_GEN_FOOTPRINT_POSITION( PCB_EDIT_FRAME* aEditFrame ) :
+        DIALOG_GEN_FOOTPRINT_POSITION_BASE( aEditFrame ),
+        m_editFrame( aEditFrame ),
+        m_job( nullptr )
 {
-public:
-    DIALOG_GEN_FOOTPRINT_POSITION( PCB_EDIT_FRAME * aParent ):
+    m_messagesPanel->SetFileName( Prj().GetProjectPath() + wxT( "report.txt" ) );
+    m_reporter = &m_messagesPanel->Reporter();
+    initDialog();
+
+    SetupStandardButtons( { { wxID_OK, _( "Generate Position File" ) },
+                            { wxID_CANCEL, _( "Close" ) } } );
+
+    GetSizer()->SetSizeHints( this );
+    Centre();
+}
+
+
+
+DIALOG_GEN_FOOTPRINT_POSITION::DIALOG_GEN_FOOTPRINT_POSITION( JOB_EXPORT_PCB_POS* aJob,
+                                                              PCB_EDIT_FRAME*     aEditFrame,
+                                                              wxWindow*           aParent ) :
         DIALOG_GEN_FOOTPRINT_POSITION_BASE( aParent ),
-        m_parent( aParent )
-    {
-        m_messagesPanel->SetFileName( Prj().GetProjectPath() + wxT( "report.txt" ) );
-        m_reporter = &m_messagesPanel->Reporter();
-        initDialog();
+        m_editFrame( aEditFrame ),
+        m_job( aJob )
+{
+    m_messagesPanel->Hide();
+    initDialog();
 
-        SetupStandardButtons( { { wxID_OK,     _( "Generate Position File" ) },
-                                { wxID_CANCEL, _( "Close" )                  } } );
-
-        GetSizer()->SetSizeHints(this);
-        Centre();
-    }
-
-private:
-    void initDialog();
-    void OnOutputDirectoryBrowseClicked( wxCommandEvent& event ) override;
-    void OnGenerate( wxCommandEvent& event ) override;
-
-    void onUpdateUIUnits( wxUpdateUIEvent& event ) override
-    {
-        m_radioBoxUnits->Enable( m_rbFormat->GetSelection() != 2 );
-    }
-
-    void onUpdateUIFileOpt( wxUpdateUIEvent& event ) override
-    {
-        m_radioBoxFilesCount->Enable( m_rbFormat->GetSelection() != 2 );
-    }
-
-    void onUpdateUIOnlySMD( wxUpdateUIEvent& event ) override
-    {
-        if( m_rbFormat->GetSelection() == 2 )
-        {
-            m_onlySMD->SetValue( false );
-            m_onlySMD->Enable( false );
-        }
-        else
-        {
-            m_onlySMD->Enable( true );
-        }
-    }
-
-    void onUpdateUInegXcoord( wxUpdateUIEvent& event ) override
-    {
-        if( m_rbFormat->GetSelection() == 2 )
-        {
-            m_negateXcb->SetValue( false );
-            m_negateXcb->Enable( false );
-        }
-        else
-        {
-            m_negateXcb->Enable( true );
-        }
-    }
-
-    void onUpdateUIExcludeTH( wxUpdateUIEvent& event ) override
-    {
-        if( m_rbFormat->GetSelection() == 2 )
-        {
-            m_excludeTH->SetValue( false );
-            m_excludeTH->Enable( false );
-        }
-        else
-        {
-            m_excludeTH->Enable( true );
-        }
-    }
-
-    void onUpdateUIincludeBoardEdge( wxUpdateUIEvent& event ) override
-    {
-        m_cbIncludeBoardEdge->Enable( m_rbFormat->GetSelection() == 2 );
-    }
-
-    /**
-     * Creates files in text or csv format
-     */
-    bool CreateAsciiFiles();
-
-    /**
-     * Creates placement files in gerber format
-     */
-    bool CreateGerberFiles();
-
-    // accessors to options:
-    bool UnitsMM()
-    {
-        return m_radioBoxUnits->GetSelection() == 1;
-    }
-
-    bool OneFileOnly()
-    {
-        return m_radioBoxFilesCount->GetSelection() == 1;
-    }
-
-    bool OnlySMD()
-    {
-        return m_onlySMD->GetValue();
-    }
-
-    bool ExcludeAllTH()
-    {
-        return m_excludeTH->GetValue();
-    }
-
-    bool ExcludeDNP()
-    {
-        return m_excludeDNP->GetValue();
-    }
-
-private:
-    PCB_EDIT_FRAME* m_parent;
-    REPORTER*       m_reporter;
-};
+    SetupStandardButtons();
 
+    GetSizer()->SetSizeHints( this );
+    Centre();
+}
 
 
 void DIALOG_GEN_FOOTPRINT_POSITION::initDialog()
 {
-    m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
+    if( !m_job )
+    {
+        m_browseButton->SetBitmap( KiBitmapBundle( BITMAPS::small_folder ) );
 
-    PROJECT_FILE&    projectFile = m_parent->Prj().GetProjectFile();
-    PCBNEW_SETTINGS* cfg = m_parent->GetPcbNewSettings();
+        PROJECT_FILE&    projectFile = m_editFrame->Prj().GetProjectFile();
+        PCBNEW_SETTINGS* cfg = m_editFrame->GetPcbNewSettings();
 
-    m_units = cfg->m_PlaceFile.units == 0 ? EDA_UNITS::INCHES : EDA_UNITS::MILLIMETRES;
+        m_units = cfg->m_PlaceFile.units == 0 ? EDA_UNITS::INCHES : EDA_UNITS::MILLIMETRES;
 
-    // Output directory
-    if( !projectFile.m_PcbLastPath[ LAST_PATH_POS_FILES ].IsEmpty() )
-        m_outputDirectoryName->SetValue( projectFile.m_PcbLastPath[ LAST_PATH_POS_FILES ] );
+        // Output directory
+        m_outputDirectoryName->SetValue( projectFile.m_PcbLastPath[LAST_PATH_POS_FILES] );
+
+        // Update Options
+        m_unitsCtrl->SetSelection( cfg->m_PlaceFile.units );
+        m_singleFile->SetValue( cfg->m_PlaceFile.file_options == 1 );
+        m_formatCtrl->SetSelection( cfg->m_PlaceFile.file_format );
+        m_cbIncludeBoardEdge->SetValue( cfg->m_PlaceFile.include_board_edge );
+        m_useDrillPlaceOrigin->SetValue( cfg->m_PlaceFile.use_aux_origin );
+        m_onlySMD->SetValue( cfg->m_PlaceFile.only_SMD );
+        m_negateXcb->SetValue( cfg->m_PlaceFile.negate_xcoord );
+        m_excludeTH->SetValue( cfg->m_PlaceFile.exclude_TH );
+
+        // Update sizes and sizers:
+        m_messagesPanel->MsgPanelSetMinSize( wxSize( -1, 160 ) );
+    }
     else
-        m_outputDirectoryName->SetValue( cfg->m_PlaceFile.output_directory );
+    {
+        SetTitle( m_job->GetSettingsDialogTitle() );
 
-    // Update Options
-    m_radioBoxUnits->SetSelection( cfg->m_PlaceFile.units );
-    m_radioBoxFilesCount->SetSelection( cfg->m_PlaceFile.file_options );
-    m_rbFormat->SetSelection( cfg->m_PlaceFile.file_format );
-    m_cbIncludeBoardEdge->SetValue( cfg->m_PlaceFile.include_board_edge );
-    m_useDrillPlaceOrigin->SetValue( cfg->m_PlaceFile.use_aux_origin );
-    m_onlySMD->SetValue( cfg->m_PlaceFile.only_SMD );
-    m_negateXcb->SetValue( cfg->m_PlaceFile.negate_xcoord );
-    m_excludeTH->SetValue( cfg->m_PlaceFile.exclude_TH );
+        m_browseButton->Hide();
+
+        m_units = m_job->m_units == JOB_EXPORT_PCB_POS::UNITS::INCHES ? EDA_UNITS::INCHES
+																	  : EDA_UNITS::MILLIMETRES;
+
+        m_staticTextDir->SetLabel( _( "Output file:" ) );
+        m_outputDirectoryName->SetValue( m_job->GetConfiguredOutputPath() );
+
+        m_unitsCtrl->SetSelection( static_cast<int>( m_job->m_units ) );
+        m_singleFile->SetValue( m_job->m_singleFile );
+        m_formatCtrl->SetSelection( static_cast<int>( m_job->m_format ) );
+        m_cbIncludeBoardEdge->SetValue( m_job->m_gerberBoardEdge );
+		m_useDrillPlaceOrigin->SetValue( m_job->m_useDrillPlaceFileOrigin );
+        m_onlySMD->SetValue( m_job->m_smdOnly );
+        m_negateXcb->SetValue( m_job->m_negateBottomX );
+        m_excludeTH->SetValue( m_job->m_excludeFootprintsWithTh );
+        m_excludeDNP->SetValue( m_job->m_excludeDNP );
+
+        m_messagesPanel->Hide();
+    }
+
+    // DIALOG_SHIM needs a unique hash_key because classname will be the same for both job and
+    // non-job versions (which have different sizes).
+    m_hash_key = TO_UTF8( GetTitle() );
 
-    // Update sizes and sizers:
-    m_messagesPanel->MsgPanelSetMinSize( wxSize( -1, 160 ) );
     GetSizer()->SetSizeHints( this );
 }
 
 
-void DIALOG_GEN_FOOTPRINT_POSITION::OnOutputDirectoryBrowseClicked( wxCommandEvent& event )
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUIUnits( wxUpdateUIEvent& event )
+{
+    m_unitsLabel->Enable( m_formatCtrl->GetSelection() != 2 );
+    m_unitsCtrl->Enable( m_formatCtrl->GetSelection() != 2 );
+}
+
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUIFileOpt( wxUpdateUIEvent& event )
+{
+    m_singleFile->Enable( m_formatCtrl->GetSelection() != 2 );
+}
+
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUIOnlySMD( wxUpdateUIEvent& event )
+{
+    if( m_formatCtrl->GetSelection() == 2 )
+    {
+        m_onlySMD->SetValue( false );
+        m_onlySMD->Enable( false );
+    }
+    else
+    {
+        m_onlySMD->Enable( true );
+    }
+}
+
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUInegXcoord( wxUpdateUIEvent& event )
+{
+    if( m_formatCtrl->GetSelection() == 2 )
+    {
+        m_negateXcb->SetValue( false );
+        m_negateXcb->Enable( false );
+    }
+    else
+    {
+        m_negateXcb->Enable( true );
+    }
+}
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUIExcludeTH( wxUpdateUIEvent& event )
+{
+    if( m_formatCtrl->GetSelection() == 2 )
+    {
+        m_excludeTH->SetValue( false );
+        m_excludeTH->Enable( false );
+    }
+    else
+    {
+        m_excludeTH->Enable( true );
+    }
+}
+
+
+bool DIALOG_GEN_FOOTPRINT_POSITION::UnitsMM()
+{
+    return m_unitsCtrl->GetSelection() == 1;
+}
+
+
+bool DIALOG_GEN_FOOTPRINT_POSITION::OneFileOnly()
+{
+    return m_singleFile->GetValue();
+}
+
+
+bool DIALOG_GEN_FOOTPRINT_POSITION::OnlySMD()
+{
+    return m_onlySMD->GetValue();
+}
+
+
+bool DIALOG_GEN_FOOTPRINT_POSITION::ExcludeAllTH()
+{
+    return m_excludeTH->GetValue();
+}
+
+
+bool DIALOG_GEN_FOOTPRINT_POSITION::ExcludeDNP()
+{
+    return m_excludeDNP->GetValue();
+}
+
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onUpdateUIincludeBoardEdge( wxUpdateUIEvent& event )
+{
+    m_cbIncludeBoardEdge->Enable( m_formatCtrl->GetSelection() == 2 );
+}
+
+
+void DIALOG_GEN_FOOTPRINT_POSITION::onOutputDirectoryBrowseClicked( wxCommandEvent& event )
 {
     // Build the absolute path of current output directory to preselect it in the file browser.
     wxString path = ExpandEnvVarSubstitutions( m_outputDirectoryName->GetValue(), &Prj() );
@@ -217,7 +249,7 @@ void DIALOG_GEN_FOOTPRINT_POSITION::OnOutputDirectoryBrowseClicked( wxCommandEve
 
     if( dialog.ShowModal() == wxID_YES )
     {
-        wxString boardFilePath = ( (wxFileName) m_parent->GetBoard()->GetFileName() ).GetPath();
+        wxString boardFilePath = ( (wxFileName) m_editFrame->GetBoard()->GetFileName() ).GetPath();
 
         if( !dirName.MakeRelativeTo( boardFilePath ) )
         {
@@ -231,37 +263,57 @@ void DIALOG_GEN_FOOTPRINT_POSITION::OnOutputDirectoryBrowseClicked( wxCommandEve
 }
 
 
-void DIALOG_GEN_FOOTPRINT_POSITION::OnGenerate( wxCommandEvent& event )
+void DIALOG_GEN_FOOTPRINT_POSITION::onGenerate( wxCommandEvent& event )
 {
-    m_units  = m_radioBoxUnits->GetSelection() == 0 ? EDA_UNITS::INCHES : EDA_UNITS::MILLIMETRES;
+    if( !m_job )
+    {
+        m_units  = m_unitsCtrl->GetSelection() == 0 ? EDA_UNITS::INCHES : EDA_UNITS::MILLIMETRES;
 
-    PCBNEW_SETTINGS* cfg = m_parent->GetPcbNewSettings();
+        PCBNEW_SETTINGS* cfg = m_editFrame->GetPcbNewSettings();
 
-    wxString dirStr = m_outputDirectoryName->GetValue();
-    // Keep unix directory format convention in cfg files
-    dirStr.Replace( wxT( "\\" ), wxT( "/" ) );
+        wxString dirStr = m_outputDirectoryName->GetValue();
+        // Keep unix directory format convention in cfg files
+        dirStr.Replace( wxT( "\\" ), wxT( "/" ) );
 
-    m_parent->Prj().GetProjectFile().m_PcbLastPath[LAST_PATH_POS_FILES] = dirStr;
-    cfg->m_PlaceFile.output_directory   = dirStr;
-    cfg->m_PlaceFile.units              = m_units == EDA_UNITS::INCHES ? 0 : 1;
-    cfg->m_PlaceFile.file_options       = m_radioBoxFilesCount->GetSelection();
-    cfg->m_PlaceFile.file_format        = m_rbFormat->GetSelection();
-    cfg->m_PlaceFile.include_board_edge = m_cbIncludeBoardEdge->GetValue();
-    cfg->m_PlaceFile.exclude_TH         = m_excludeTH->GetValue();
-    cfg->m_PlaceFile.only_SMD           = m_onlySMD->GetValue();
-    cfg->m_PlaceFile.use_aux_origin     = m_useDrillPlaceOrigin->GetValue();
-    cfg->m_PlaceFile.negate_xcoord      = m_negateXcb->GetValue();
+        m_editFrame->Prj().GetProjectFile().m_PcbLastPath[LAST_PATH_POS_FILES] = dirStr;
+        cfg->m_PlaceFile.output_directory   = dirStr;
+        cfg->m_PlaceFile.units              = m_units == EDA_UNITS::INCHES ? 0 : 1;
+        cfg->m_PlaceFile.file_options       = m_singleFile->GetValue() ? 1 : 0;
+        cfg->m_PlaceFile.file_format        = m_formatCtrl->GetSelection();
+        cfg->m_PlaceFile.include_board_edge = m_cbIncludeBoardEdge->GetValue();
+        cfg->m_PlaceFile.exclude_TH         = m_excludeTH->GetValue();
+        cfg->m_PlaceFile.only_SMD           = m_onlySMD->GetValue();
+        cfg->m_PlaceFile.use_aux_origin     = m_useDrillPlaceOrigin->GetValue();
+        cfg->m_PlaceFile.negate_xcoord      = m_negateXcb->GetValue();
 
-    if( m_rbFormat->GetSelection() == 2 )
-        CreateGerberFiles();
+        if( m_formatCtrl->GetSelection() == 2 )
+            CreateGerberFiles();
+        else
+            CreateAsciiFiles();
+    }
     else
-        CreateAsciiFiles();
+    {
+        m_job->SetConfiguredOutputPath( m_outputDirectoryName->GetValue() );
+        m_job->m_units = m_unitsCtrl->GetSelection() == 0 ? JOB_EXPORT_PCB_POS::UNITS::INCHES
+                                                          : JOB_EXPORT_PCB_POS::UNITS::MILLIMETERS;
+        m_job->m_format = static_cast<JOB_EXPORT_PCB_POS::FORMAT>( m_formatCtrl->GetSelection() );
+        m_job->m_side = JOB_EXPORT_PCB_POS::SIDE::BOTH;
+        m_job->m_singleFile = m_singleFile->GetValue();
+        m_job->m_gerberBoardEdge = m_cbIncludeBoardEdge->GetValue();
+        m_job->m_excludeFootprintsWithTh = m_excludeTH->GetValue();
+        m_job->m_smdOnly = m_onlySMD->GetValue();
+        m_job->m_useDrillPlaceFileOrigin = m_useDrillPlaceOrigin->GetValue();
+        m_job->m_negateBottomX = m_negateXcb->GetValue();
+        m_job->m_excludeDNP = m_excludeDNP->GetValue();
+
+        event.Skip();   // Allow normal close action
+    }
 }
 
 
 bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
 {
-    BOARD*     brd = m_parent->GetBoard();
+    BOARD*     brd = m_editFrame->GetBoard();
     wxString   msg;
     int        fullcount = 0;
 
@@ -272,15 +324,15 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
             [&]( wxString* token ) -> bool
             {
                 // Handles board->GetTitleBlock() *and* board->GetProject()
-                return m_parent->GetBoard()->ResolveTextVar( token, 0 );
+                return m_editFrame->GetBoard()->ResolveTextVar( token, 0 );
             };
 
-    wxString path = m_parent->GetPcbNewSettings()->m_PlaceFile.output_directory;
+    wxString path = m_editFrame->GetPcbNewSettings()->m_PlaceFile.output_directory;
     path = ExpandTextVars( path, &textResolver );
     path = ExpandEnvVarSubstitutions( path, nullptr );
 
     wxFileName  outputDir = wxFileName::DirName( path );
-    wxString    boardFilename = m_parent->GetBoard()->GetFileName();
+    wxString   boardFilename = m_editFrame->GetBoard()->GetFileName();
 
     m_reporter = &m_messagesPanel->Reporter();
 
@@ -291,7 +343,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
         return false;
     }
 
-    wxFileName fn = m_parent->GetBoard()->GetFileName();
+    wxFileName fn = m_editFrame->GetBoard()->GetFileName();
     fn.SetPath( outputDir.GetPath() );
 
     // Create the Front and Top side placement files. Gerber P&P files are always separated.
@@ -310,7 +362,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
     }
 
     msg.Printf( _( "Front (top side) placement file: '%s'." ), filename );
-    m_reporter->Report( msg, RPT_SEVERITY_INFO );
+    m_reporter->Report( msg, RPT_SEVERITY_ACTION );
 
     msg.Printf( _( "Component count: %d." ), fpcount );
     m_reporter->Report( msg, RPT_SEVERITY_INFO );
@@ -332,7 +384,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
 
     // Display results
     msg.Printf( _( "Back (bottom side) placement file: '%s'." ), filename );
-    m_reporter->Report( msg, RPT_SEVERITY_INFO );
+    m_reporter->Report( msg, RPT_SEVERITY_ACTION );
 
     msg.Printf( _( "Component count: %d." ), fpcount );
     m_reporter->Report( msg, RPT_SEVERITY_INFO );
@@ -341,7 +393,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
     msg.Printf( _( "Full component count: %d." ), fullcount );
     m_reporter->Report( msg, RPT_SEVERITY_INFO );
 
-    m_reporter->Report( _( "File generation successful." ), RPT_SEVERITY_INFO );
+    m_reporter->Report( _( "Done." ), RPT_SEVERITY_INFO );
 
     return true;
 }
@@ -349,10 +401,10 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateGerberFiles()
 
 bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
 {
-    BOARD *    brd = m_parent->GetBoard();
+    BOARD*     brd = m_editFrame->GetBoard();
     wxString   msg;
     bool       singleFile = OneFileOnly();
-    bool       useCSVfmt = m_rbFormat->GetSelection() == 1;
+    bool       useCSVfmt = m_formatCtrl->GetSelection() == 1;
     bool       useAuxOrigin = m_useDrillPlaceOrigin->GetValue();
     int        fullcount = 0;
     int        topSide = true;
@@ -379,15 +431,15 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
             [&]( wxString* token ) -> bool
             {
                 // Handles board->GetTitleBlock() *and* board->GetProject()
-                return m_parent->GetBoard()->ResolveTextVar( token, 0 );
+                return m_editFrame->GetBoard()->ResolveTextVar( token, 0 );
             };
 
-    wxString path = m_parent->GetPcbNewSettings()->m_PlaceFile.output_directory;
+    wxString path = m_editFrame->GetPcbNewSettings()->m_PlaceFile.output_directory;
     path = ExpandTextVars( path, &textResolver );
     path = ExpandEnvVarSubstitutions( path, nullptr );
 
-    wxFileName  outputDir = wxFileName::DirName( path );
-    wxString    boardFilename = m_parent->GetBoard()->GetFileName();
+    wxFileName outputDir = wxFileName::DirName( path );
+    wxString   boardFilename = m_editFrame->GetBoard()->GetFileName();
 
     m_reporter = &m_messagesPanel->Reporter();
 
@@ -398,35 +450,23 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
         return false;
     }
 
-    wxFileName fn = m_parent->GetBoard()->GetFileName();
+    wxFileName fn = m_editFrame->GetBoard()->GetFileName();
     fn.SetPath( outputDir.GetPath() );
 
     // Create the Front or Top side placement file, or a single file
     topSide = true;
-    bottomSide = false;
-
-    if( singleFile )
-    {
-        bottomSide = true;
-        fn.SetName( fn.GetName() + wxT( "-" ) + wxT( "all" ) );
-    }
-    else
-    {
-        fn.SetName( fn.GetName() + wxT( "-" ) + PLACE_FILE_EXPORTER::GetFrontSideName().c_str() );
-    }
+    bottomSide = singleFile;
 
+    fn.SetName( PLACE_FILE_EXPORTER::DecorateFilename( fn.GetName(), topSide, bottomSide ) );
+    fn.SetExt( FILEEXT::FootprintPlaceFileExtension );
 
     if( useCSVfmt )
     {
         fn.SetName( fn.GetName() + wxT( "-" ) + FILEEXT::FootprintPlaceFileExtension );
         fn.SetExt( wxT( "csv" ) );
     }
-    else
-    {
-        fn.SetExt( FILEEXT::FootprintPlaceFileExtension );
-    }
 
-    int fpcount = m_parent->DoGenFootprintsPositionFile( fn.GetFullPath(), UnitsMM(), OnlySMD(),
+    int fpcount = m_editFrame->DoGenFootprintsPositionFile( fn.GetFullPath(), UnitsMM(), OnlySMD(),
                                                          ExcludeAllTH(), ExcludeDNP(), topSide,
                                                          bottomSide, useCSVfmt, useAuxOrigin,
                                                          negateBottomX );
@@ -443,14 +483,14 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
     else
         msg.Printf( _( "Front (top side) placement file: '%s'." ), fn.GetFullPath() );
 
-    m_reporter->Report( msg, RPT_SEVERITY_INFO );
+    m_reporter->Report( msg, RPT_SEVERITY_ACTION );
 
     msg.Printf( _( "Component count: %d." ), fpcount );
     m_reporter->Report( msg, RPT_SEVERITY_INFO );
 
     if( singleFile  )
     {
-        m_reporter->Report( _( "File generation successful." ), RPT_SEVERITY_INFO );
+        m_reporter->Report( _( "Done." ), RPT_SEVERITY_INFO );
         return true;
     }
 
@@ -460,22 +500,19 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
     bottomSide = true;
     fn = brd->GetFileName();
     fn.SetPath( outputDir.GetPath() );
-    fn.SetName( fn.GetName() + wxT( "-" ) + PLACE_FILE_EXPORTER::GetBackSideName().c_str() );
+    fn.SetName( PLACE_FILE_EXPORTER::DecorateFilename( fn.GetName(), topSide, bottomSide ) );
+    fn.SetExt( FILEEXT::FootprintPlaceFileExtension );
 
     if( useCSVfmt )
     {
         fn.SetName( fn.GetName() + wxT( "-" ) + FILEEXT::FootprintPlaceFileExtension );
         fn.SetExt( wxT( "csv" ) );
     }
-    else
-    {
-        fn.SetExt( FILEEXT::FootprintPlaceFileExtension );
-    }
 
-    fpcount = m_parent->DoGenFootprintsPositionFile( fn.GetFullPath(), UnitsMM(), OnlySMD(),
-                                                     ExcludeAllTH(), ExcludeDNP(), topSide,
-                                                     bottomSide, useCSVfmt,
-                                                     useAuxOrigin, negateBottomX );
+    fpcount = m_editFrame->DoGenFootprintsPositionFile( fn.GetFullPath(), UnitsMM(), OnlySMD(),
+                                                        ExcludeAllTH(), ExcludeDNP(), topSide,
+                                                        bottomSide, useCSVfmt,
+                                                        useAuxOrigin, negateBottomX );
 
     if( fpcount < 0 )
     {
@@ -489,7 +526,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
     if( !singleFile )
     {
         msg.Printf( _( "Back (bottom side) placement file: '%s'." ), fn.GetFullPath() );
-        m_reporter->Report( msg, RPT_SEVERITY_INFO );
+        m_reporter->Report( msg, RPT_SEVERITY_ACTION );
 
         msg.Printf( _( "Component count: %d." ), fpcount );
         m_reporter->Report( msg, RPT_SEVERITY_INFO );
@@ -502,7 +539,7 @@ bool DIALOG_GEN_FOOTPRINT_POSITION::CreateAsciiFiles()
         m_reporter->Report( msg, RPT_SEVERITY_INFO );
     }
 
-    m_reporter->Report( _( "File generation successful." ), RPT_SEVERITY_INFO );
+    m_reporter->Report( _( "Done." ), RPT_SEVERITY_INFO );
     return true;
 }
 
