@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2007 Dick Hollenbeck, dick@softplc.com
- * Copyright (C) 2015-2022 KiCad Developers, see change_log.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,244 +30,287 @@
 #include <drc/drc_rule.h>
 #include <board.h>
 #include <pcb_marker.h>
+#include <i18n_utility.h>
 
 
-// These, being statically-defined, require specialized I18N handling.  We continue to
-// use the _() macro so that string harvesting by the I18N framework doesn't have to be
-// specialized, but we don't translate on initialization and instead do it in the getters.
-
-#undef _
-#define _(s) s
+// These, being statically-defined, require specialized I18N handling.
+// We don't translate on initialization and instead do it in the getters.
 
 // NOTE: Avoid changing the settings key for a DRC item after it has been created
 
 
-DRC_ITEM DRC_ITEM::heading_electrical( 0, _( "Electrical" ), "" );
-DRC_ITEM DRC_ITEM::heading_DFM( 0, _( "Design for Manufacturing" ), "" );
-DRC_ITEM DRC_ITEM::heading_schematic_parity( 0, _( "Schematic Parity" ), "" );
-DRC_ITEM DRC_ITEM::heading_signal_integrity( 0, _( "Signal Integrity" ), "" );
-DRC_ITEM DRC_ITEM::heading_readability( 0, _( "Readability" ), "" );
-DRC_ITEM DRC_ITEM::heading_misc( 0, _( "Miscellaneous" ), "" );
+DRC_ITEM DRC_ITEM::heading_electrical( 0, _HKI( "Electrical" ), "" );
+DRC_ITEM DRC_ITEM::heading_DFM( 0, _HKI( "Design for Manufacturing" ), "" );
+DRC_ITEM DRC_ITEM::heading_schematic_parity( 0, _HKI( "Schematic Parity" ), "" );
+DRC_ITEM DRC_ITEM::heading_signal_integrity( 0, _HKI( "Signal Integrity" ), "" );
+DRC_ITEM DRC_ITEM::heading_readability( 0, _HKI( "Readability" ), "" );
+DRC_ITEM DRC_ITEM::heading_misc( 0, _HKI( "Miscellaneous" ), "" );
+DRC_ITEM DRC_ITEM::heading_internal( 0, "", "" );
+DRC_ITEM DRC_ITEM::heading_deprecated( 0, "", "" );
 
 DRC_ITEM DRC_ITEM::unconnectedItems( DRCE_UNCONNECTED_ITEMS,
-        _( "Missing connection between items" ),
+        _HKI( "Missing connection between items" ),
         wxT( "unconnected_items" ) );
 
 DRC_ITEM DRC_ITEM::shortingItems( DRCE_SHORTING_ITEMS,
-        _( "Items shorting two nets" ),
+        _HKI( "Items shorting two nets" ),
         wxT( "shorting_items" ) );
 
 DRC_ITEM DRC_ITEM::itemsNotAllowed( DRCE_ALLOWED_ITEMS,
-        _( "Items not allowed" ),
+        _HKI( "Items not allowed" ),
         wxT( "items_not_allowed" ) );
 
 DRC_ITEM DRC_ITEM::textOnEdgeCuts( DRCE_TEXT_ON_EDGECUTS,
-        _( "Text (or dimension) on Edge.Cuts layer" ),
+        _HKI( "Text (or dimension) on Edge.Cuts layer" ),
         wxT( "text_on_edge_cuts" ) );
 
 DRC_ITEM DRC_ITEM::clearance( DRCE_CLEARANCE,
-        _( "Clearance violation" ),
+        _HKI( "Clearance violation" ),
         wxT( "clearance" ) );
 
+DRC_ITEM DRC_ITEM::creepage( DRCE_CREEPAGE,
+        _HKI( "Creepage violation" ),
+        wxT( "creepage" ) );
+
 DRC_ITEM DRC_ITEM::tracksCrossing( DRCE_TRACKS_CROSSING,
-        _( "Tracks crossing" ),
+        _HKI( "Tracks crossing" ),
         wxT( "tracks_crossing" ) );
 
 DRC_ITEM DRC_ITEM::edgeClearance( DRCE_EDGE_CLEARANCE,
-        _( "Board edge clearance violation" ),
+        _HKI( "Board edge clearance violation" ),
         wxT( "copper_edge_clearance" ) );
 
 DRC_ITEM DRC_ITEM::zonesIntersect( DRCE_ZONES_INTERSECT,
-        _( "Copper zones intersect" ),
+        _HKI( "Copper zones intersect" ),
         wxT( "zones_intersect" ) );
 
 DRC_ITEM DRC_ITEM::isolatedCopper( DRCE_ISOLATED_COPPER,
-        _( "Isolated copper fill" ),
+        _HKI( "Isolated copper fill" ),
         wxT( "isolated_copper" ) );
 
 DRC_ITEM DRC_ITEM::starvedThermal( DRCE_STARVED_THERMAL,
-        _( "Thermal relief connection to zone incomplete" ),
+        _HKI( "Thermal relief connection to zone incomplete" ),
         wxT( "starved_thermal" ) );
 
 DRC_ITEM DRC_ITEM::viaDangling( DRCE_DANGLING_VIA,
-        _( "Via is not connected or connected on only one layer" ),
+        _HKI( "Via is not connected or connected on only one layer" ),
         wxT( "via_dangling" ) );
 
 DRC_ITEM DRC_ITEM::trackDangling( DRCE_DANGLING_TRACK,
-        _( "Track has unconnected end" ),
+        _HKI( "Track has unconnected end" ),
         wxT( "track_dangling" ) );
 
 DRC_ITEM DRC_ITEM::holeClearance( DRCE_HOLE_CLEARANCE,
-        _( "Hole clearance violation" ),
+        _HKI( "Hole clearance violation" ),
         wxT( "hole_clearance" ) );
 
 DRC_ITEM DRC_ITEM::holeNearHole( DRCE_DRILLED_HOLES_TOO_CLOSE,
-        _( "Drilled holes too close together" ),
-        wxT( "hole_near_hole" ) );
+        _HKI( "Drilled hole too close to other hole" ),
+        wxT( "hole_to_hole" ) );
 
 DRC_ITEM DRC_ITEM::holesCoLocated( DRCE_DRILLED_HOLES_COLOCATED,
-        _( "Drilled holes co-located" ),
+        _HKI( "Drilled holes co-located" ),
         wxT( "holes_co_located" ) );
 
 DRC_ITEM DRC_ITEM::connectionWidth( DRCE_CONNECTION_WIDTH,
-        _( "Copper connection too narrow" ),
+        _HKI( "Copper connection too narrow" ),
         wxT( "connection_width" ) );
 
 DRC_ITEM DRC_ITEM::trackWidth( DRCE_TRACK_WIDTH,
-        _( "Track width" ),
+        _HKI( "Track width" ),
         wxT( "track_width" ) );
 
+DRC_ITEM DRC_ITEM::trackAngle( DRCE_TRACK_ANGLE,
+        _HKI( "Track angle" ),
+        wxT( "track_angle" ) );
+
+DRC_ITEM DRC_ITEM::trackSegmentLength( DRCE_TRACK_SEGMENT_LENGTH,
+        _HKI( "Track segment length" ),
+        wxT( "track_segment_length" ) );
+
 DRC_ITEM DRC_ITEM::annularWidth( DRCE_ANNULAR_WIDTH,
-        _( "Annular width" ),
+        _HKI( "Annular width" ),
         wxT( "annular_width" ) );
 
 DRC_ITEM DRC_ITEM::drillTooSmall( DRCE_DRILL_OUT_OF_RANGE,
-        _( "Hole size out of range" ),
+        _HKI( "Hole size out of range" ),
         wxT( "drill_out_of_range" ) );
 
 DRC_ITEM DRC_ITEM::viaDiameter( DRCE_VIA_DIAMETER,
-        _( "Via diameter" ),
+        _HKI( "Via diameter" ),
         wxT( "via_diameter" ) );
 
 DRC_ITEM DRC_ITEM::padstack( DRCE_PADSTACK,
-        _( "Padstack is not valid" ),
+        _HKI( "Padstack is questionable" ),
         wxT( "padstack" ) );
 
+DRC_ITEM DRC_ITEM::padstackInvalid( DRCE_PADSTACK_INVALID,
+        _HKI( "Padstack is not valid" ),
+        wxT( "padstack_invalid" ) );
+
 DRC_ITEM DRC_ITEM::microviaDrillTooSmall( DRCE_MICROVIA_DRILL_OUT_OF_RANGE,
-        _( "Micro via hole size out of range" ),
+        _HKI( "Micro via hole size out of range" ),
         wxT( "microvia_drill_out_of_range" ) );
 
 DRC_ITEM DRC_ITEM::courtyardsOverlap( DRCE_OVERLAPPING_FOOTPRINTS,
-        _( "Courtyards overlap" ),
+        _HKI( "Courtyards overlap" ),
         wxT( "courtyards_overlap" ) );
 
 DRC_ITEM DRC_ITEM::missingCourtyard( DRCE_MISSING_COURTYARD,
-        _( "Footprint has no courtyard defined" ),
+        _HKI( "Footprint has no courtyard defined" ),
         wxT( "missing_courtyard" ) );
 
 DRC_ITEM DRC_ITEM::malformedCourtyard( DRCE_MALFORMED_COURTYARD,
-        _( "Footprint has malformed courtyard" ),
+        _HKI( "Footprint has malformed courtyard" ),
         wxT( "malformed_courtyard" ) );
 
 DRC_ITEM DRC_ITEM::pthInsideCourtyard( DRCE_PTH_IN_COURTYARD,
-        _( "PTH inside courtyard" ),
+        _HKI( "PTH inside courtyard" ),
         wxT( "pth_inside_courtyard" ) );
 
 DRC_ITEM DRC_ITEM::npthInsideCourtyard( DRCE_NPTH_IN_COURTYARD,
-        _( "NPTH inside courtyard" ),
+        _HKI( "NPTH inside courtyard" ),
         wxT( "npth_inside_courtyard" ) );
 
 DRC_ITEM DRC_ITEM::itemOnDisabledLayer( DRCE_DISABLED_LAYER_ITEM,
-        _( "Item on a disabled copper layer" ),
+        _HKI( "Item on a disabled copper layer" ),
         wxT( "item_on_disabled_layer" ) );
 
 DRC_ITEM DRC_ITEM::invalidOutline( DRCE_INVALID_OUTLINE,
-        _( "Board has malformed outline" ),
+        _HKI( "Board has malformed outline" ),
         wxT( "invalid_outline" ) );
 
 DRC_ITEM DRC_ITEM::duplicateFootprints( DRCE_DUPLICATE_FOOTPRINT,
-        _( "Duplicate footprints" ),
+        _HKI( "Duplicate footprints" ),
         wxT( "duplicate_footprints" ) );
 
 DRC_ITEM DRC_ITEM::missingFootprint( DRCE_MISSING_FOOTPRINT,
-        _( "Missing footprint" ),
+        _HKI( "Missing footprint" ),
         wxT( "missing_footprint" ) );
 
 DRC_ITEM DRC_ITEM::extraFootprint( DRCE_EXTRA_FOOTPRINT,
-        _( "Extra footprint" ),
+        _HKI( "Extra footprint" ),
         wxT( "extra_footprint" ) );
 
 DRC_ITEM DRC_ITEM::netConflict( DRCE_NET_CONFLICT,
-        _( "Pad net doesn't match schematic" ),
+        _HKI( "Pad net doesn't match schematic" ),
         wxT( "net_conflict" ) );
 
-DRC_ITEM DRC_ITEM::schematicParityIssues( DRCE_SCHEMATIC_PARITY_ISSUES,
-        _( "Footprint attributes don't match symbol" ),
+DRC_ITEM DRC_ITEM::schematicParity( DRCE_SCHEMATIC_PARITY,
+        _HKI( "Footprint attributes don't match symbol" ),
         wxT( "footprint_symbol_mismatch" ) );
 
+DRC_ITEM DRC_ITEM::footprintFilters( DRCE_FOOTPRINT_FILTERS,
+        _HKI( "Footprint doesn't match symbol's footprint filters" ),
+        wxT( "footprint_filters_mismatch" ) );
+
 DRC_ITEM DRC_ITEM::libFootprintIssues( DRCE_LIB_FOOTPRINT_ISSUES,
-        _( "Footprint not found in libraries" ),
+        _HKI( "Footprint not found in libraries" ),
         wxT( "lib_footprint_issues" ) );
 
 DRC_ITEM DRC_ITEM::libFootprintMismatch( DRCE_LIB_FOOTPRINT_MISMATCH,
-        _( "Footprint doesn't match copy in library" ),
+        _HKI( "Footprint doesn't match copy in library" ),
         wxT( "lib_footprint_mismatch" ) );
 
 DRC_ITEM DRC_ITEM::unresolvedVariable( DRCE_UNRESOLVED_VARIABLE,
-        _( "Unresolved text variable" ),
+        _HKI( "Unresolved text variable" ),
         wxT( "unresolved_variable" ) );
 
 DRC_ITEM DRC_ITEM::assertionFailure( DRCE_ASSERTION_FAILURE,
-        _( "Assertion failure" ),
+        _HKI( "Assertion failure" ),
         wxT( "assertion_failure" ) );
 
+DRC_ITEM DRC_ITEM::genericWarning( DRCE_GENERIC_WARNING,
+        _HKI( "Warning" ),
+        wxT( "generic_warning" ) );
+
+DRC_ITEM DRC_ITEM::genericError( DRCE_GENERIC_ERROR,
+        _HKI( "Error" ),
+        wxT( "generic_error" ) );
+
 DRC_ITEM DRC_ITEM::copperSliver( DRCE_COPPER_SLIVER,
-        _( "Copper sliver" ),
+        _HKI( "Copper sliver" ),
         wxT( "copper_sliver" ) );
 
 DRC_ITEM DRC_ITEM::solderMaskBridge( DRCE_SOLDERMASK_BRIDGE,
-        _( "Solder mask aperture bridges items with different nets" ),
+        _HKI( "Solder mask aperture bridges items with different nets" ),
         wxT( "solder_mask_bridge" ) );
 
 DRC_ITEM DRC_ITEM::silkClearance( DRCE_SILK_CLEARANCE,
-        _( "Silkscreen clipped by solder mask" ),
+        _HKI( "Silkscreen clipped by solder mask" ),
         wxT( "silk_over_copper" ) );
 
 DRC_ITEM DRC_ITEM::silkEdgeClearance( DRCE_SILK_EDGE_CLEARANCE,
-        _( "Silkscreen clipped by board edge" ),
+        _HKI( "Silkscreen clipped by board edge" ),
         wxT( "silk_edge_clearance" ) );
 
 DRC_ITEM DRC_ITEM::silkOverlaps( DRCE_OVERLAPPING_SILK,
-        _( "Silkscreen overlap" ),
+        _HKI( "Silkscreen overlap" ),
         wxT( "silk_overlap" ) );
 
 DRC_ITEM DRC_ITEM::textHeightOutOfRange( DRCE_TEXT_HEIGHT,
-        _( "Text height out of range" ),
+        _HKI( "Text height out of range" ),
         wxT( "text_height" ) );
 
 DRC_ITEM DRC_ITEM::textThicknessOutOfRange( DRCE_TEXT_THICKNESS,
-        _( "Text thickness out of range" ),
+        _HKI( "Text thickness out of range" ),
         wxT( "text_thickness" ) );
 
 DRC_ITEM DRC_ITEM::lengthOutOfRange( DRCE_LENGTH_OUT_OF_RANGE,
-        _( "Trace length out of range" ),
+        _HKI( "Track length out of range" ),
         wxT( "length_out_of_range" ) );
 
 DRC_ITEM DRC_ITEM::skewOutOfRange( DRCE_SKEW_OUT_OF_RANGE,
-        _( "Skew between traces out of range" ),
+        _HKI( "Skew between tracks out of range" ),
         wxT( "skew_out_of_range" ) );
 
 // Note: this used to only check against a max value, hence the settings key too_many_vias
 DRC_ITEM DRC_ITEM::viaCountOutOfRange( DRCE_VIA_COUNT_OUT_OF_RANGE,
-        _( "Too many or too few vias on a connection" ),
+        _HKI( "Too many or too few vias on a connection" ),
         wxT( "too_many_vias" ) );
 
 DRC_ITEM DRC_ITEM::diffPairGapOutOfRange( DRCE_DIFF_PAIR_GAP_OUT_OF_RANGE,
-        _( "Differential pair gap out of range" ),
+        _HKI( "Differential pair gap out of range" ),
         wxT( "diff_pair_gap_out_of_range" ) );
 
 DRC_ITEM DRC_ITEM::diffPairUncoupledLengthTooLong( DRCE_DIFF_PAIR_UNCOUPLED_LENGTH_TOO_LONG,
-        _( "Differential uncoupled length too long" ),
+        _HKI( "Differential uncoupled length too long" ),
         wxT( "diff_pair_uncoupled_length_too_long" ) );
 
 DRC_ITEM DRC_ITEM::footprint( DRCE_FOOTPRINT,
-        _( "Footprint is not valid" ),
+        _HKI( "Footprint is not valid" ),
         wxT( "footprint" ) );
 
 DRC_ITEM DRC_ITEM::footprintTypeMismatch( DRCE_FOOTPRINT_TYPE_MISMATCH,
-        _( "Footprint component type doesn't match footprint pads" ),
+        _HKI( "Footprint component type doesn't match footprint pads" ),
         wxT( "footprint_type_mismatch" ) );
 
 DRC_ITEM DRC_ITEM::footprintTHPadhasNoHole( DRCE_PAD_TH_WITH_NO_HOLE,
-        _( "Through hole pad has no hole" ),
+        _HKI( "Through hole pad has no hole" ),
         wxT( "through_hole_pad_without_hole" ) );
 
+DRC_ITEM DRC_ITEM::mirroredTextOnFrontLayer( DRCE_MIRRORED_TEXT_ON_FRONT_LAYER,
+        _HKI( "Mirrored text on front layer" ),
+        wxT( "mirrored_text_on_front_layer" ) );
 
-std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes( {
+DRC_ITEM DRC_ITEM::nonMirroredTextOnBackLayer( DRCE_NONMIRRORED_TEXT_ON_BACK_LAYER,
+        _HKI( "Non-Mirrored text on back layer" ),
+        wxT( "nonmirrored_text_on_back_layer" ) );
+
+/// Deprecated item names
+/// They have the current error code but the old name for compatibility with old settings files
+
+DRC_ITEM DRC_ITEM::holeNearHolev8( DRCE_DRILLED_HOLES_TOO_CLOSE,
+        _HKI( "Drilled hole too close to other hole - deprecated" ),
+        wxT( "hole_near_hole" ) );
+
+std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes(
+        {
             DRC_ITEM::heading_electrical,
             DRC_ITEM::shortingItems,
             DRC_ITEM::tracksCrossing,
             DRC_ITEM::clearance,
+            DRC_ITEM::creepage,
             DRC_ITEM::viaDangling,
             DRC_ITEM::trackDangling,
             DRC_ITEM::starvedThermal,
@@ -276,7 +319,10 @@ std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes( {
             DRC_ITEM::edgeClearance,
             DRC_ITEM::holeClearance,
             DRC_ITEM::holeNearHole,
+            DRC_ITEM::holesCoLocated,
             DRC_ITEM::trackWidth,
+            DRC_ITEM::trackAngle,
+            DRC_ITEM::trackSegmentLength,
             DRC_ITEM::annularWidth,
             DRC_ITEM::drillTooSmall,
             DRC_ITEM::microviaDrillTooSmall,
@@ -292,7 +338,8 @@ std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes( {
             DRC_ITEM::duplicateFootprints,
             DRC_ITEM::missingFootprint,
             DRC_ITEM::extraFootprint,
-            DRC_ITEM::schematicParityIssues,
+            DRC_ITEM::schematicParity,
+            DRC_ITEM::footprintFilters,
             DRC_ITEM::netConflict,
             DRC_ITEM::unconnectedItems,
 
@@ -309,6 +356,8 @@ std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes( {
             DRC_ITEM::silkEdgeClearance,
             DRC_ITEM::textHeightOutOfRange,
             DRC_ITEM::textThicknessOutOfRange,
+            DRC_ITEM::mirroredTextOnFrontLayer,
+            DRC_ITEM::nonMirroredTextOnBackLayer,
 
             DRC_ITEM::heading_misc,
             DRC_ITEM::itemsNotAllowed,
@@ -320,11 +369,23 @@ std::vector<std::reference_wrapper<RC_ITEM>> DRC_ITEM::allItemTypes( {
             DRC_ITEM::npthInsideCourtyard,
             DRC_ITEM::itemOnDisabledLayer,
             DRC_ITEM::unresolvedVariable,
-
             DRC_ITEM::footprintTypeMismatch,
             DRC_ITEM::libFootprintIssues,
             DRC_ITEM::libFootprintMismatch,
-            DRC_ITEM::footprintTHPadhasNoHole
+            DRC_ITEM::footprintTHPadhasNoHole,
+
+            // Deprecated items need to come second to last in the list
+            // They will not be shown in the panel but will be used to
+            // parse old settings files
+            DRC_ITEM::heading_deprecated,
+            DRC_ITEM::holeNearHolev8,
+
+            // DRC_ITEM types with no user-editable severities
+            // NOTE: this MUST be the last grouping in the list!
+            DRC_ITEM::heading_internal,
+            DRC_ITEM::padstackInvalid,
+            DRC_ITEM::genericError,
+            DRC_ITEM::genericWarning,
         } );
 
 
@@ -337,6 +398,7 @@ std::shared_ptr<DRC_ITEM> DRC_ITEM::Create( int aErrorCode )
     case DRCE_ALLOWED_ITEMS:            return std::make_shared<DRC_ITEM>( itemsNotAllowed );
     case DRCE_TEXT_ON_EDGECUTS:         return std::make_shared<DRC_ITEM>( textOnEdgeCuts );
     case DRCE_CLEARANCE:                return std::make_shared<DRC_ITEM>( clearance );
+    case DRCE_CREEPAGE:                 return std::make_shared<DRC_ITEM>( creepage );
     case DRCE_TRACKS_CROSSING:          return std::make_shared<DRC_ITEM>( tracksCrossing );
     case DRCE_EDGE_CLEARANCE:           return std::make_shared<DRC_ITEM>( edgeClearance );
     case DRCE_ZONES_INTERSECT:          return std::make_shared<DRC_ITEM>( zonesIntersect );
@@ -349,10 +411,13 @@ std::shared_ptr<DRC_ITEM> DRC_ITEM::Create( int aErrorCode )
     case DRCE_HOLE_CLEARANCE:           return std::make_shared<DRC_ITEM>( holeClearance );
     case DRCE_CONNECTION_WIDTH:         return std::make_shared<DRC_ITEM>( connectionWidth );
     case DRCE_TRACK_WIDTH:              return std::make_shared<DRC_ITEM>( trackWidth );
+    case DRCE_TRACK_ANGLE:              return std::make_shared<DRC_ITEM>( trackAngle );
+    case DRCE_TRACK_SEGMENT_LENGTH:     return std::make_shared<DRC_ITEM>( trackSegmentLength );
     case DRCE_ANNULAR_WIDTH:            return std::make_shared<DRC_ITEM>( annularWidth );
     case DRCE_DRILL_OUT_OF_RANGE:       return std::make_shared<DRC_ITEM>( drillTooSmall );
     case DRCE_VIA_DIAMETER:             return std::make_shared<DRC_ITEM>( viaDiameter );
     case DRCE_PADSTACK:                 return std::make_shared<DRC_ITEM>( padstack );
+    case DRCE_PADSTACK_INVALID:         return std::make_shared<DRC_ITEM>( padstackInvalid );
     case DRCE_MICROVIA_DRILL_OUT_OF_RANGE: return std::make_shared<DRC_ITEM>( microviaDrillTooSmall );
     case DRCE_OVERLAPPING_FOOTPRINTS:   return std::make_shared<DRC_ITEM>( courtyardsOverlap );
     case DRCE_MISSING_COURTYARD:        return std::make_shared<DRC_ITEM>( missingCourtyard );
@@ -365,11 +430,14 @@ std::shared_ptr<DRC_ITEM> DRC_ITEM::Create( int aErrorCode )
     case DRCE_DUPLICATE_FOOTPRINT:      return std::make_shared<DRC_ITEM>( duplicateFootprints );
     case DRCE_NET_CONFLICT:             return std::make_shared<DRC_ITEM>( netConflict );
     case DRCE_EXTRA_FOOTPRINT:          return std::make_shared<DRC_ITEM>( extraFootprint );
-    case DRCE_SCHEMATIC_PARITY_ISSUES:  return std::make_shared<DRC_ITEM>( schematicParityIssues );
+    case DRCE_SCHEMATIC_PARITY:         return std::make_shared<DRC_ITEM>( schematicParity );
+    case DRCE_FOOTPRINT_FILTERS:        return std::make_shared<DRC_ITEM>( footprintFilters );
     case DRCE_LIB_FOOTPRINT_ISSUES:     return std::make_shared<DRC_ITEM>( libFootprintIssues );
     case DRCE_LIB_FOOTPRINT_MISMATCH:   return std::make_shared<DRC_ITEM>( libFootprintMismatch );
     case DRCE_UNRESOLVED_VARIABLE:      return std::make_shared<DRC_ITEM>( unresolvedVariable );
     case DRCE_ASSERTION_FAILURE:        return std::make_shared<DRC_ITEM>( assertionFailure );
+    case DRCE_GENERIC_WARNING:          return std::make_shared<DRC_ITEM>( genericWarning );
+    case DRCE_GENERIC_ERROR:            return std::make_shared<DRC_ITEM>( genericError );
     case DRCE_COPPER_SLIVER:            return std::make_shared<DRC_ITEM>( copperSliver );
     case DRCE_OVERLAPPING_SILK:         return std::make_shared<DRC_ITEM>( silkOverlaps );
     case DRCE_SILK_CLEARANCE:           return std::make_shared<DRC_ITEM>( silkClearance );
@@ -385,6 +453,8 @@ std::shared_ptr<DRC_ITEM> DRC_ITEM::Create( int aErrorCode )
     case DRCE_FOOTPRINT:                return std::make_shared<DRC_ITEM>( footprint );
     case DRCE_FOOTPRINT_TYPE_MISMATCH:  return std::make_shared<DRC_ITEM>( footprintTypeMismatch );
     case DRCE_PAD_TH_WITH_NO_HOLE:      return std::make_shared<DRC_ITEM>( footprintTHPadhasNoHole );
+    case DRCE_MIRRORED_TEXT_ON_FRONT_LAYER:        return std::make_shared<DRC_ITEM>( mirroredTextOnFrontLayer );
+    case DRCE_NONMIRRORED_TEXT_ON_BACK_LAYER:      return std::make_shared<DRC_ITEM>( nonMirroredTextOnBackLayer );
     default:
         wxFAIL_MSG( wxT( "Unknown DRC error code" ) );
         return nullptr;
