@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2023 KiCad Developers.
+ * Copyright The KiCad Developers.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,6 +18,7 @@
  */
 
 
+#include <advanced_config.h>
 #include <board.h>
 #include <board_design_settings.h>
 #include <pcb_track.h>
@@ -179,8 +180,8 @@ static bool commonParallelProjection( const PCB_ARC& p, const PCB_ARC& n, SHAPE_
             clip_total_angle = p_end_angle - n_start_angle;
     }
 
-    // One arc starts exactly where the other ends
-    if( clip_total_angle == ANGLE_0 )
+    // One arc starts approximately where the other ends
+    if( clip_total_angle <= EDA_ANGLE( ADVANCED_CFG::GetCfg().m_MinParallelAngle ) )
         return false;
 
     VECTOR2I n_start_pt = n_center + VECTOR2I( KiROUND( n_radius ), 0 );
@@ -192,6 +193,10 @@ static bool commonParallelProjection( const PCB_ARC& p, const PCB_ARC& n, SHAPE_
     pClip = SHAPE_ARC( p_center, p_start_pt, clip_total_angle );
     nClip = SHAPE_ARC( n_center, n_start_pt, clip_total_angle );
 
+    if( std::abs( pClip.GetP0().x - pClip.GetArcMid().x ) < 10
+     || std::abs( nClip.GetP0().x - nClip.GetArcMid().x ) < 10 )
+        return false;
+
     return true;
 }
 
@@ -239,7 +244,9 @@ struct DIFF_PAIR_COUPLED_SEGMENTS
     SHAPE_ARC    coupledArcP;
     PCB_TRACK*   parentN;
     PCB_TRACK*   parentP;
-    int          computedGap;
+    int64_t      computedGap;
+    VECTOR2I     nearestN;
+    VECTOR2I     nearestP;
     PCB_LAYER_ID layer;
     bool         couplingFailMin;
     bool         couplingFailMax;
@@ -271,8 +278,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
         PCB_TRACK* sp = dyn_cast<PCB_TRACK*>( itemP );
-        std::optional<DIFF_PAIR_COUPLED_SEGMENTS> bestCoupled;
-        int bestGap = std::numeric_limits<int>::max();
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
 
         if( !sp )
             continue;
@@ -292,7 +298,7 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 
             // Segments that are ~ 1 IU in length per side are approximately parallel (tolerance is 1 IU)
             // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
-            if( ssp.SquaredLength() > 2 && ssn.SquaredLength() > 2 && ssp.ApproxParallel(ssn) )
+            if( ssp.SquaredLength() > 2 && ssn.SquaredLength() > 2 && !ssp.Intersect( ssn, false, true ) )
             {
                 DIFF_PAIR_COUPLED_SEGMENTS cpair;
                 bool coupled = commonParallelProjection( ssp, ssn, cpair.coupledP, cpair.coupledN );
@@ -302,33 +308,33 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
                     cpair.parentP = sp;
                     cpair.parentN = sn;
                     cpair.layer = sp->GetLayer();
-                    cpair.computedGap = (cpair.coupledP.A - cpair.coupledN.A).EuclideanNorm();
+                    cpair.coupledP.NearestPoints( cpair.coupledN, cpair.nearestP, cpair.nearestN,
+                                                  cpair.computedGap );
+                    cpair.computedGap = std::sqrt( cpair.computedGap );  // NearestPoints returns squared distance
                     cpair.computedGap -= ( sp->GetWidth() + sn->GetWidth() ) / 2;
-
-                    if( cpair.computedGap < bestGap )
-                    {
-                        bestGap = cpair.computedGap;
-                        bestCoupled = cpair;
-                    }
+                    coupled_vec.push_back( cpair );
                 }
 
             }
         }
 
-        if( bestCoupled )
+        for( auto coupled : coupled_vec )
         {
             auto excludeSelf = [&]( BOARD_ITEM* aItem )
             {
-                if( aItem == bestCoupled->parentN || aItem == bestCoupled->parentP )
+                if( aItem == coupled->parentN || aItem == coupled->parentP )
                     return false;
 
                 if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T
                     || aItem->Type() == PCB_ARC_T )
                 {
-                    BOARD_CONNECTED_ITEM* bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
+                    PCB_TRACK* bci = static_cast<PCB_TRACK*>( aItem );
 
-                    if( bci->GetNetCode() == bestCoupled->parentN->GetNetCode()
-                        || bci->GetNetCode() == bestCoupled->parentP->GetNetCode() )
+                    // Directly connected items don't count
+                    if( bci->HitTest( coupled->coupledN.A, 0 )
+                        || bci->HitTest( coupled->coupledN.B, 0 )
+                        || bci->HitTest( coupled->coupledP.A, 0 )
+                        || bci->HitTest( coupled->coupledP.B, 0 ) )
                     {
                         return false;
                     }
@@ -337,26 +343,21 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
                 return true;
             };
 
-            SHAPE_SEGMENT checkSegStart( bestCoupled->coupledP.A, bestCoupled->coupledN.A );
-            SHAPE_SEGMENT checkSegEnd( bestCoupled->coupledP.B, bestCoupled->coupledN.B );
-            DRC_RTREE*    tree = bestCoupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
+            SHAPE_SEGMENT checkSeg( coupled->nearestN, coupled->nearestP );
+            DRC_RTREE*    tree = coupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
 
             // check if there's anything in between the segments suspected to be coupled. If
             // there's nothing, assume they are really coupled.
 
-            if( !tree->CheckColliding( &checkSegStart, sp->GetLayer(), 0, excludeSelf )
-                  && !tree->CheckColliding( &checkSegEnd, sp->GetLayer(), 0, excludeSelf ) )
-            {
-                aDp.coupled.push_back( *bestCoupled );
-            }
+            if( !tree->CheckColliding( &checkSeg, sp->GetLayer(), 0, excludeSelf ) )
+                aDp.coupled.push_back( *coupled );
         }
     }
 
     for( BOARD_CONNECTED_ITEM* itemP : aDp.itemsP )
     {
         PCB_ARC* sp = dyn_cast<PCB_ARC*>( itemP );
-        std::optional<DIFF_PAIR_COUPLED_SEGMENTS> bestCoupled;
-        int bestGap = std::numeric_limits<int>::max();
+        std::vector<std::optional<DIFF_PAIR_COUPLED_SEGMENTS>> coupled_vec;
 
         if( !sp )
             continue;
@@ -373,7 +374,9 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
 
             // Segments that are ~ 1 IU in length per side are approximately parallel (tolerance is 1 IU)
             // with everything and their parallel projection is < 1 IU, leading to bad distance calculations
-            if( sp->GetLength() > 2 && sn->GetLength() > 2 && ( sp->GetCenter() - sn->GetCenter() ).SquaredEuclideanNorm() < 4 )
+            int64_t sqWidth = static_cast<int64_t>( sp->GetWidth() ) * sp->GetWidth();
+
+            if( sp->GetLength() > 2 && sn->GetLength() > 2 && sp->GetCenter().SquaredDistance( sn->GetCenter() ) < sqWidth )
             {
                 DIFF_PAIR_COUPLED_SEGMENTS cpair;
                 cpair.isArc = true;
@@ -384,54 +387,45 @@ static void extractDiffPairCoupledItems( DIFF_PAIR_ITEMS& aDp )
                     cpair.parentP = sp;
                     cpair.parentN = sn;
                     cpair.layer = sp->GetLayer();
-                    cpair.computedGap = KiROUND( std::abs( cpair.coupledArcP.GetRadius()
-                                                 - cpair.coupledArcN.GetRadius() ) );
+                    cpair.coupledArcP.NearestPoints( cpair.coupledArcN, cpair.nearestP, cpair.nearestN,
+                                                  cpair.computedGap );
+                    cpair.computedGap = std::sqrt( cpair.computedGap );  // NearestPoints returns squared distance
                     cpair.computedGap -= ( sp->GetWidth() + sn->GetWidth() ) / 2;
-
-                    if( cpair.computedGap < bestGap )
-                    {
-                        bestGap = cpair.computedGap;
-                        bestCoupled = cpair;
-                    }
+                    coupled_vec.push_back( cpair );
                 }
-
             }
         }
 
-        if( bestCoupled )
+        for( auto coupled : coupled_vec )
         {
             auto excludeSelf =
                     [&] ( BOARD_ITEM *aItem )
                     {
-                        if( aItem == bestCoupled->parentN || aItem == bestCoupled->parentP )
-                        {
+                        if( aItem == coupled->parentN || aItem == coupled->parentP )
                             return false;
-                        }
 
                         if( aItem->Type() == PCB_TRACE_T || aItem->Type() == PCB_VIA_T || aItem->Type() == PCB_ARC_T )
                         {
-                            auto bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
+                            BOARD_CONNECTED_ITEM* bci = static_cast<BOARD_CONNECTED_ITEM*>( aItem );
 
-                            if( bci->GetNetCode() == bestCoupled->parentN->GetNetCode()
-                            ||  bci->GetNetCode() == bestCoupled->parentP->GetNetCode() )
+                            if( bci->GetNetCode() == coupled->parentN->GetNetCode()
+                                    ||  bci->GetNetCode() == coupled->parentP->GetNetCode() )
+                            {
                                 return false;
+                            }
                         }
 
                         return true;
                     };
 
-            SHAPE_SEGMENT checkSegStart( bestCoupled->coupledP.A, bestCoupled->coupledN.A );
-            SHAPE_SEGMENT checkSegEnd( bestCoupled->coupledP.B, bestCoupled->coupledN.B );
-            DRC_RTREE*    tree = bestCoupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
+            SHAPE_SEGMENT checkArcMid( coupled->coupledArcN.GetArcMid(), coupled->coupledArcP.GetArcMid() );
+            DRC_RTREE*    tree = coupled->parentP->GetBoard()->m_CopperItemRTreeCache.get();
 
             // check if there's anything in between the segments suspected to be coupled. If
             // there's nothing, assume they are really coupled.
 
-            if( !tree->CheckColliding( &checkSegStart, sp->GetLayer(), 0, excludeSelf )
-                  && !tree->CheckColliding( &checkSegEnd, sp->GetLayer(), 0, excludeSelf ) )
-            {
-                aDp.coupled.push_back( *bestCoupled );
-            }
+            if( !tree->CheckColliding( &checkArcMid, sp->GetLayer(), 0, excludeSelf ) )
+                aDp.coupled.push_back( *coupled );
         }
     }
 }
@@ -457,26 +451,22 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                 {
                     drc_dbg( 10, wxT( "eval dp %p\n" ), item );
 
-                    const DRC_CONSTRAINT_T constraintsToCheck[] = {
-                            DIFF_PAIR_GAP_CONSTRAINT,
-                            MAX_UNCOUPLED_CONSTRAINT
-                    };
-
-                    for( int i = 0; i < 2; i++ )
+                    for( DRC_CONSTRAINT_T constraintType : { DIFF_PAIR_GAP_CONSTRAINT,
+                                                             MAX_UNCOUPLED_CONSTRAINT } )
                     {
-                        DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( constraintsToCheck[ i ],
+                        DRC_CONSTRAINT constraint = m_drcEngine->EvalRules( constraintType,
                                                                             item, nullptr,
                                                                             item->GetLayer() );
 
                         if( constraint.IsNull() || constraint.GetSeverity() == RPT_SEVERITY_IGNORE )
                             continue;
 
-                        drc_dbg( 10, wxT( "cns %d item %p\n" ), constraintsToCheck[i], item );
+                        drc_dbg( 10, wxT( "cns %d item %p\n" ), (int) constraintType, item );
 
                         DRC_RULE* parentRule = constraint.GetParentRule();
                         wxString ruleName = parentRule ? parentRule->m_Name : constraint.GetName();
 
-                        switch( constraintsToCheck[i] )
+                        switch( constraintType )
                         {
                         case DIFF_PAIR_GAP_CONSTRAINT:
                             key.gapConstraint = constraint.GetValue();
@@ -535,22 +525,29 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
 
         drc_dbg(10, wxT( "       coupled prims : %d\n" ), (int) itemSet.coupled.size() );
 
+        std::set<BOARD_CONNECTED_ITEM*> allItems;
+
         for( BOARD_CONNECTED_ITEM* item : itemSet.itemsN )
         {
             if( PCB_TRACK* track = dynamic_cast<PCB_TRACK*>( item ) )
-                itemSet.totalLengthN += track->GetLength();
+            {
+                if( allItems.insert( item ).second)
+                    itemSet.totalLengthN += track->GetLength();
+            }
         }
 
         for( BOARD_CONNECTED_ITEM* item : itemSet.itemsP )
         {
             if( PCB_TRACK* track = dynamic_cast<PCB_TRACK*>( item ) )
-                itemSet.totalLengthP += track->GetLength();
+            {
+                if( allItems.insert( item ).second)
+                    itemSet.totalLengthP += track->GetLength();
+            }
         }
 
         for( DIFF_PAIR_COUPLED_SEGMENTS& dp : itemSet.coupled )
         {
-            int length = dp.coupledN.Length();
-
+            int length = dp.isArc ? dp.coupledArcN.GetLength() : dp.coupledN.Length();
             wxCHECK2( dp.parentN && dp.parentP, continue );
 
             std::shared_ptr<KIGFX::VIEW_OVERLAY> overlay = m_drcEngine->GetDebugOverlay();
@@ -566,10 +563,10 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                 overlay->Line( dp.coupledN );
             }
 
-            drc_dbg( 10, wxT( "               len %d gap %d l %d\n" ),
+            drc_dbg( 10, wxT( "               len %d gap %ld l %d\n" ),
                      length,
-                     dp.computedGap,
-                     dp.parentP->GetLayer() );
+                     (long int) dp.computedGap,
+                     (int) dp.parentP->GetLayer() );
 
             if( key.gapConstraint )
             {
@@ -597,8 +594,7 @@ bool test::DRC_TEST_PROVIDER_DIFF_PAIR_COUPLING::Run()
                                      MessageTextFromValue( itemSet.totalCoupled ),
                                      MessageTextFromValue( totalLen ) ) );
 
-        int totalUncoupled = totalLen - itemSet.totalCoupled;
-
+        int  totalUncoupled = totalLen - itemSet.totalCoupled;
         bool uncoupledViolation = false;
 
         if( key.uncoupledConstraint && ( !itemSet.itemsP.empty() || !itemSet.itemsN.empty() ) )
