@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 Thomas Pointhuber <thomas.pointhuber@gmx.at>
- * Copyright (C) 2020-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,19 +28,26 @@
 
 #include <wx/string.h>
 
+#include <font/fontconfig.h>
 #include <pcb_io_altium_designer.h>
 #include <altium_pcb.h>
+#include <altium_pcb_compound_file.h>
 #include <io/io_utils.h>
 #include <io/altium/altium_binary_parser.h>
 #include <pcb_io/pcb_io.h>
+#include <reporter.h>
 
 #include <board.h>
 
 #include <compoundfilereader.h>
 #include <utf.h>
 
-PCB_IO_ALTIUM_DESIGNER::PCB_IO_ALTIUM_DESIGNER() : PCB_IO( wxS( "Altium Designer" ) )
+PCB_IO_ALTIUM_DESIGNER::PCB_IO_ALTIUM_DESIGNER() :
+        PCB_IO( wxS( "Altium Designer" ) )
 {
+    m_reporter = &WXLOG_REPORTER::GetInstance();
+
+    RegisterCallback( PCB_IO_ALTIUM_DESIGNER::DefaultLayerMappingCallback );
 }
 
 
@@ -49,6 +56,21 @@ PCB_IO_ALTIUM_DESIGNER::~PCB_IO_ALTIUM_DESIGNER()
 }
 
 
+std::map<wxString, PCB_LAYER_ID> PCB_IO_ALTIUM_DESIGNER::DefaultLayerMappingCallback(
+        const std::vector<INPUT_LAYER_DESC>& aInputLayerDescriptionVector )
+{
+    std::map<wxString, PCB_LAYER_ID> retval;
+
+    // Just return a the auto-mapped layers
+    for( INPUT_LAYER_DESC layerDesc : aInputLayerDescriptionVector )
+    {
+        retval.insert( { layerDesc.Name, layerDesc.AutoMapLayer } );
+    }
+
+    return retval;
+}
+
+
 bool PCB_IO_ALTIUM_DESIGNER::checkFileHeader( const wxString& aFileName )
 {
     // Compound File Binary Format header
@@ -75,12 +97,15 @@ bool PCB_IO_ALTIUM_DESIGNER::CanReadLibrary( const wxString& aFileName ) const
 
 
 BOARD* PCB_IO_ALTIUM_DESIGNER::LoadBoard( const wxString& aFileName, BOARD* aAppendToMe,
-                                          const STRING_UTF8_MAP* aProperties, PROJECT* aProject )
+                                          const std::map<std::string, UTF8>* aProperties, PROJECT* aProject )
 {
+
     m_props = aProperties;
 
     m_board = aAppendToMe ? aAppendToMe : new BOARD();
 
+    fontconfig::FONTCONFIG::SetReporter( &WXLOG_REPORTER::GetInstance() );
+
     // Give the filename to the board if it's new
     if( !aAppendToMe )
         m_board->SetFileName( aFileName );
@@ -111,12 +136,12 @@ BOARD* PCB_IO_ALTIUM_DESIGNER::LoadBoard( const wxString& aFileName, BOARD* aApp
     };
     // clang-format on
 
-    ALTIUM_COMPOUND_FILE altiumPcbFile( aFileName );
+    ALTIUM_PCB_COMPOUND_FILE altiumPcbFile( aFileName );
 
     try
     {
         // Parse File
-        ALTIUM_PCB pcb( m_board, m_progressReporter );
+        ALTIUM_PCB pcb( m_board, m_progressReporter, m_layer_mapping_handler, m_reporter );
         pcb.Parse( altiumPcbFile, mapping );
     }
     catch( CFB::CFBException& exception )
@@ -151,6 +176,8 @@ long long PCB_IO_ALTIUM_DESIGNER::GetLibraryTimestamp( const wxString& aLibraryP
 
 void PCB_IO_ALTIUM_DESIGNER::loadAltiumLibrary( const wxString& aLibraryPath )
 {
+    fontconfig::FONTCONFIG::SetReporter( nullptr );
+
     try
     {
         auto it = m_fplibFiles.find( aLibraryPath );
@@ -161,18 +188,24 @@ void PCB_IO_ALTIUM_DESIGNER::loadAltiumLibrary( const wxString& aLibraryPath )
         if( aLibraryPath.Lower().EndsWith( wxS( ".pcblib" ) ) )
         {
             m_fplibFiles[aLibraryPath].emplace_back(
-                    std::make_unique<ALTIUM_COMPOUND_FILE>( aLibraryPath ) );
+                    std::make_unique<ALTIUM_PCB_COMPOUND_FILE>( aLibraryPath ) );
         }
         else if( aLibraryPath.Lower().EndsWith( wxS( ".intlib" ) ) )
         {
-            std::unique_ptr<ALTIUM_COMPOUND_FILE> intCom =
-                    std::make_unique<ALTIUM_COMPOUND_FILE>( aLibraryPath );
+            std::unique_ptr<ALTIUM_PCB_COMPOUND_FILE> intCom =
+                    std::make_unique<ALTIUM_PCB_COMPOUND_FILE>( aLibraryPath );
 
             std::map<wxString, const CFB::COMPOUND_FILE_ENTRY*> pcbLibFiles =
                     intCom->EnumDir( L"PCBLib" );
-
             for( const auto& [pcbLibName, pcbCfe] : pcbLibFiles )
-                m_fplibFiles[aLibraryPath].push_back( intCom->DecodeIntLibStream( *pcbCfe ) );
+            {
+                auto decodedStream = intCom->DecodeIntLibStream( *pcbCfe );
+                m_fplibFiles[aLibraryPath].push_back(
+                    std::unique_ptr<ALTIUM_PCB_COMPOUND_FILE>(
+                        static_cast<ALTIUM_PCB_COMPOUND_FILE*>(decodedStream.release())
+                    )
+                );
+            }
         }
     }
     catch( CFB::CFBException& exception )
@@ -184,7 +217,7 @@ void PCB_IO_ALTIUM_DESIGNER::loadAltiumLibrary( const wxString& aLibraryPath )
 
 void PCB_IO_ALTIUM_DESIGNER::FootprintEnumerate( wxArrayString&  aFootprintNames,
                                                  const wxString& aLibraryPath, bool aBestEfforts,
-                                                 const STRING_UTF8_MAP* aProperties )
+                                                 const std::map<std::string, UTF8>* aProperties )
 {
     loadAltiumLibrary( aLibraryPath );
 
@@ -214,6 +247,22 @@ void PCB_IO_ALTIUM_DESIGNER::FootprintEnumerate( wxArrayString&  aFootprintNames
             std::map<wxString, wxString> properties = parser.ReadProperties();
 
             uint32_t numberOfFootprints = parser.Read<uint32_t>();
+
+            // The number of footprints might be bogus. Following it is a list of index strings to
+            // find the footprints, and this list might actually be longer than how many footprints
+            // actually are in the library.
+            // If this case was detected, truncate the list to the number of footprints.
+            // Sample in #18452 shows that by simply truncating can get the correct footprints.
+            // Fixes https://gitlab.com/kicad/code/kicad/issues/18452.
+            // After truncation we shall no longer detect if the stream is fully parsed.
+            bool footprintListNotTruncated = true;
+
+            if ( patternMap.size() < numberOfFootprints )
+            {
+                numberOfFootprints = patternMap.size();
+                footprintListNotTruncated = false;
+            }
+
             aFootprintNames.Alloc( numberOfFootprints );
 
             for( size_t i = 0; i < numberOfFootprints; i++ )
@@ -224,6 +273,7 @@ void PCB_IO_ALTIUM_DESIGNER::FootprintEnumerate( wxArrayString&  aFootprintNames
                 wxString           fpPattern( charBuffer, wxConvISO8859_1 );
 
                 auto it = patternMap.find( fpPattern );
+
                 if( it != patternMap.end() )
                 {
                     aFootprintNames.Add( it->second ); // Proper unicode name
@@ -243,7 +293,7 @@ void PCB_IO_ALTIUM_DESIGNER::FootprintEnumerate( wxArrayString&  aFootprintNames
                                                   FormatPath( streamName ) ) );
             }
 
-            if( parser.GetRemainingBytes() != 0 )
+            if( footprintListNotTruncated && parser.GetRemainingBytes() != 0 )
             {
                 THROW_IO_ERROR( wxString::Format( "%s stream is not fully parsed",
                                                   FormatPath( streamName ) ) );
@@ -259,7 +309,7 @@ void PCB_IO_ALTIUM_DESIGNER::FootprintEnumerate( wxArrayString&  aFootprintNames
 
 FOOTPRINT* PCB_IO_ALTIUM_DESIGNER::FootprintLoad( const wxString& aLibraryPath,
                                                   const wxString& aFootprintName, bool aKeepUUID,
-                                                  const STRING_UTF8_MAP* aProperties )
+                                                  const std::map<std::string, UTF8>* aProperties )
 {
     loadAltiumLibrary( aLibraryPath );
 
@@ -270,15 +320,16 @@ FOOTPRINT* PCB_IO_ALTIUM_DESIGNER::FootprintLoad( const wxString& aLibraryPath,
 
     try
     {
-        for( std::unique_ptr<ALTIUM_COMPOUND_FILE>& altiumLibFile : it->second )
+        for( std::unique_ptr<ALTIUM_PCB_COMPOUND_FILE>& altiumLibFile : it->second )
         {
+            altiumLibFile->CacheLibModels();
             auto [dirName, fpCfe] = altiumLibFile->FindLibFootprintDirName( aFootprintName );
 
             if( dirName.IsEmpty() )
                 continue;
 
             // Parse File
-            ALTIUM_PCB pcb( m_board, nullptr, aLibraryPath, aFootprintName );
+            ALTIUM_PCB pcb( m_board, nullptr, m_layer_mapping_handler, m_reporter, aLibraryPath, aFootprintName );
             return pcb.ParseFootprint( *altiumLibFile, aFootprintName );
         }
     }
@@ -290,4 +341,4 @@ FOOTPRINT* PCB_IO_ALTIUM_DESIGNER::FootprintLoad( const wxString& aLibraryPath,
     THROW_IO_ERROR( wxString::Format( _( "Footprint '%s' not found in '%s'." ),
                                       aFootprintName,
                                       aLibraryPath ) );
-}
\ No newline at end of file
+}
